/*
 *
 * Copyright (C) 2001 MontaVista Software, ppopov@mvista.com
 * Copied and modified Carsten Langgaard's time.c
 *
 * Carsten Langgaard, carstenl@mips.com
 * Copyright (C) 1999,2000 MIPS Technologies, Inc.  All rights reserved.
 *
 * ########################################################################
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * ########################################################################
 *
 * Setting up the clock on the Alchemy boards.
 *
 * Update.  Always configure the kernel with CONFIG_NEW_TIME_C.  This
 * will use the user interface gettimeofday() functions from the
 * arch/mips/kernel/time.c, and we provide the clock interrupt processing
 * and the timer offset compute functions. -- Dan
 */

/*
 * See discussion on kernel timer in Docuemtation/mips/Alchemy.README
 */

#include <linux/types.h>
#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel_stat.h>
#include <linux/sched.h>
#include <linux/spinlock.h>
#include <linux/kernel.h>
#include <linux/fcntl.h>
#include <linux/miscdevice.h>
#include <linux/module.h>
#include <linux/hardirq.h>
#include <linux/fs.h>

#include <asm/compiler.h>
#include <asm/mipsregs.h>
#include <asm/ptrace.h>
#include <asm/time.h>
#include <asm/debug.h>
#include <asm/time.h>
#include <asm/uaccess.h>

#include <asm/mach-au1x00/au1000.h>

#include <linux/timex.h>

/* I haven't found anyone that doesn't use a 12 MHz source clock,
 * but just in case.....
 */
#ifndef CONFIG_AU1000_SRC_CLK
#define CONFIG_AU1000_SRC_CLK 12000000
#endif

extern void startup_match20_interrupt(void);
void	(*au1k_wait_ptr)(void);


#ifndef CONFIG_AU1000_32KHZ_CLK
#define CONFIG_AU1000_32KHZ_CLK 32768
#endif

/* I haven't found anyone that doesn't use a 12 MHz source clock,
 * but just in case.....
 */
#ifdef CONFIG_AU1000_SRC_CLK
#define AU1000_SRC_CLK	CONFIG_AU1000_SRC_CLK
#else
#define AU1000_SRC_CLK	12000000
#endif


static DEFINE_SPINLOCK(time_lock);
static u32 r4k_offset; /* Amount to increment compare reg each time */
static u32 r4k_cur;    /* What counter should be at next timer irq */
/* Cycle counter value at the previous timer interrupt.. */
static unsigned int timerhi = 0, timerlo = 0;


#ifdef CONFIG_SOC_AU1X00_32KHZ_TIMER
/* Use the integrated RTC timer as the system timer tick. RTC (counter1)
match1 is used as the tick source */

static volatile u32 rtcm1_offset; /* Amount to increment match reg each time */
static volatile u32 rtcm1_cur;    /* What counter should be at next timer irq */
static volatile u32 rtcm1_last;   /* What match/counter was at at the last timer irq */
static volatile u32 rtcm1_drift;

irqreturn_t rtcm1_irq(int irq, void *dev_id, struct pt_regs *regs)
{
	/* This handler mirrors mips_timer_interrupt() below, except that
	this handler is called by do_IRQ() */
	static int jiffie_drift = 0;
	int num_do_timer = 0;
	u32 rtc;

	/* disable all other IRQs. without this, this handler can be
	interrupted which allows the RTC to advance *WAY* ahead of the RTCM1
	value which causes the kernel to hang, waiting for rollover. */
	spin_lock_irq(&time_lock);

#ifndef CONFIG_SMP
	update_process_times(user_mode(regs));
#endif
	/* check if RTC counter has rolled-over, but rtcm1_cur has not */
	rtc = au_readl(SYS_RTCREAD);
	while (rtc < rtcm1_cur) {
		++num_do_timer;
		rtcm1_last = rtcm1_cur;
		rtcm1_cur += rtcm1_offset;
	}
	while (rtcm1_cur <= rtc) {
		++num_do_timer;
		rtcm1_last = rtcm1_cur;
		rtcm1_cur += rtcm1_offset;
		/* check if rtcm1_cur has rolled over, and thus ahead of rtc */
		if (rtcm1_last > rtcm1_cur)
			break;
	}
	jiffie_drift += num_do_timer;
	if ((rtcm1_drift != 0) && (jiffie_drift >= rtcm1_drift)) {
		jiffie_drift -= rtcm1_drift;
		++num_do_timer;
	}

	/* Check to make sure that the counter is not at the value we're about
	to program into the match; if so, adjust accordingly. */
	rtc = au_readl(SYS_RTCREAD);

	/* Check to make sure that the value programmed into RTCM1 is no more
	than rtcm1_offset ticks away. This is complicated by the need to handle
	32-bit rollover of the counter, and the fact that the RTC may be ahead
	of RTCM1. This value represents the number of RTC ticks needed for this
	code to read the current RTC value, make any decisions, and write
	to RTCMATCH1. */
#define RTCM1_UPDATE_DELAY 8
	rtc += RTCM1_UPDATE_DELAY;

	if (rtcm1_cur > rtc) {
		if ((rtcm1_cur - rtc) < (2 * rtcm1_offset))
			/* rtcm1_cur is just ahead of rtc */
			au_writel(rtcm1_cur, SYS_RTCMATCH1); /* normal update */
		else
			/* rtc has rolled-over, ahead of rtcm1_cur */
			au_writel(rtc + 2, SYS_RTCMATCH1); /* special update */
	} else {
		/* rtc > rtcm1_cur */
		if ((rtc - rtcm1_cur) < (2 * rtcm1_offset))
			/* rtc is just ahead of rtcm1_cur */
			au_writel(rtc + 2, SYS_RTCMATCH1); /* special update */
		else
			/* rtcm1_cur has rolled-over, ahead of rtc */
			au_writel(rtcm1_cur, SYS_RTCMATCH1); /* normal update */
	}

	while (num_do_timer-- > 0)
		do_timer(regs);

	/* SYS_CNTRCTRL_RM1 shouldn't need to be examined since this register is
	written no faster than HZ rate, which is plenty long for the updates to
	occur. Nonetheless, in practice this is needed, but by doing it at the
	end, where the update likely should have already occured, we allow the
	update to RTCM1 to occur in parallel with the calls to do_timer().
	This also guarantees that the next rtcm1_irq() write to RTCMATCH1 can
	occur without polling. */
	while (au_readl(SYS_COUNTER_CNTRL) & SYS_CNTRCTRL_RM1);

	spin_unlock_irq(&time_lock);
	return IRQ_HANDLED;
}

static void setup_rtcm1_interrupt(void)
{
	/* Setup match1 to interrupt HZ times per second.*/
	rtcm1_offset = CONFIG_AU1000_32KHZ_CLK / HZ;
	if ((rtcm1_offset * HZ) != CONFIG_AU1000_32KHZ_CLK)
		rtcm1_offset += 1; /* round up */
	/* Determine the number of ticks until we are one tick off the real rtc
	due to rounding of rtcm1_offset. */
	rtcm1_drift = CONFIG_AU1000_32KHZ_CLK / (rtcm1_offset * HZ - CONFIG_AU1000_32KHZ_CLK);
	rtcm1_cur = au_readl(SYS_RTCREAD) + rtcm1_offset;
	while (au_readl(SYS_COUNTER_CNTRL) & SYS_CNTRCTRL_RM1);
	au_writel(rtcm1_cur, SYS_RTCMATCH1);
	while (au_readl(SYS_COUNTER_CNTRL) & SYS_CNTRCTRL_RM1);
}

static unsigned long do_fast_rtcm1_gettimeoffset(void)
{
	unsigned long rtc, offset;
	static u32 num_huge_offsets = 0;

	rtc = au_readl(SYS_RTCREAD);
	if (rtc < rtcm1_last) /* RTC has rolled-over */
		offset = (0 - rtcm1_last) + rtc;
	else
		offset = rtc - rtcm1_last;

	/* NOTE: The huge offsets are unavoidable since Linux's interrupt
	response time is highly irregular. */
	if (offset > 2*rtcm1_offset) {
		if (++num_huge_offsets > 100) {
			printk("huge offset %x, rtc %x, rtcm1_last %x rtcm1 %x rtcm1_cur %x\n",
			(int)offset, (int)rtc, rtcm1_last, au_readl(SYS_RTCMATCH1), rtcm1_cur);
			num_huge_offsets = 0;
		}
	}

	/* translate the offset to microseconds by multiplying with 30.5175...
	which is identical to 15625/512 */
	offset = (unsigned long)(((u64)offset * (u64)15625) >> 9);
	return offset;
}
#endif /* CONFIG_SOC_AU1X00_32KHZ_TIMER */

#ifdef CONFIG_SOC_AU1X00_32KHZ
static unsigned long rtc_au1xxx_get_time(void)
{
	/* The TOY counter simply contains the number of seconds elapsed.
	This is interpreted from the kernel viewpoint as the number of
	seconds elapsed starting in 1970. */
	return au_readl(SYS_TOYREAD);
	}

static int rtc_au1xxx_set_time(unsigned long t)
{
	/* Take the kernel's value and record it in the counter. There is
	no need to spin on TS since this register is written infrequently. */
	au_writel(t, SYS_TOYWRITE);
	return 0;
	}

static unsigned long rtc_au1xxx_get_toym2(void)
{
	return au_readl(SYS_TOYMATCH2);
	}

static int rtc_au1xxx_set_toym2(unsigned long t)
{
	au_writel(t, SYS_TOYMATCH2);
	return 0;
	}

/********************************************************************
The following is the /dev/rtc driver that utilizes the TOY.
This code a blatant pilfer of ds1742.c */

#define AU1XXX_RTC_VERSION "1.0"
#define RTC_IS_OPEN		0x1

static unsigned long rtc_status = 0;
static spinlock_t rtc_lock;

extern void to_tm(unsigned long tim, struct rtc_time * tm);

static int rtc_au1xxx_ioctl(struct inode *inode, struct file *file,
				unsigned int cmd, unsigned long arg)
{
	struct rtc_time rtc_tm;
	ulong curr_time;

	switch (cmd) {
	case RTC_RD_TIME:	/* Read the time/date from TOY */
		curr_time = rtc_au1xxx_get_time();
		memset(&rtc_tm, 0, sizeof(struct rtc_time));
		to_tm(curr_time, &rtc_tm);
		rtc_tm.tm_year -= 1900;
		return copy_to_user((void *) arg, &rtc_tm, sizeof(rtc_tm)) ?
			-EFAULT : 0;
		break;

	case RTC_SET_TIME:
		if (!capable(CAP_SYS_TIME))
			return -EACCES;

		if (copy_from_user(&rtc_tm,
				   (struct rtc_time *) arg,
		                   sizeof(struct rtc_time)))
			return -EFAULT;

		curr_time = mktime(rtc_tm.tm_year + 1900,
				   rtc_tm.tm_mon + 1, /* tm_mon starts from 0 */
				   rtc_tm.tm_mday,
				   rtc_tm.tm_hour,
				   rtc_tm.tm_min,
				   rtc_tm.tm_sec);
		return rtc_au1xxx_set_time(curr_time);
		break;

	case RTC_WKALM_RD:	/* Read the wakeup alarm time/date from TOY M2 */
		curr_time = rtc_au1xxx_get_toym2();
		memset(&rtc_tm, 0, sizeof(struct rtc_time));
		to_tm(curr_time, &rtc_tm);
		rtc_tm.tm_year -= 1900;
		return copy_to_user((void *) arg, &rtc_tm, sizeof(rtc_tm)) ?
			-EFAULT : 0;
		break;

	case RTC_WKALM_SET:
		if (!capable(CAP_SYS_TIME))
			return -EACCES;

		if (copy_from_user(&rtc_tm,
				   (struct rtc_time *) arg,
		                   sizeof(struct rtc_time)))
			return -EFAULT;

		curr_time = mktime(rtc_tm.tm_year + 1900,
				   rtc_tm.tm_mon + 1, /* tm_mon starts from 0 */
				   rtc_tm.tm_mday,
				   rtc_tm.tm_hour,
				   rtc_tm.tm_min,
				   rtc_tm.tm_sec);
		return rtc_au1xxx_set_toym2(curr_time);
		break;

	default:
		return -EINVAL;
	}
}

static int rtc_au1xxx_open(struct inode *inode, struct file *file)
{
	int err = 0;

	spin_lock_irq(&rtc_lock);

	if (rtc_status & RTC_IS_OPEN)
		err = -EBUSY;
	else {
		/* If oscillator isn't running, then don't open() */
		if (!(au_readl(SYS_COUNTER_CNTRL) & SYS_CNTRCTRL_32S))
			err = -EIO;
		else
			rtc_status |= RTC_IS_OPEN;
}

	spin_unlock_irq(&rtc_lock);

	return err;
}

static int rtc_au1xxx_release(struct inode *inode, struct file *file)
{
	spin_lock_irq(&rtc_lock);
	rtc_status &= ~RTC_IS_OPEN;
	spin_unlock_irq(&rtc_lock);
	return 0;
}

static struct file_operations rtc_au1xxx_fops = {
	owner:THIS_MODULE,
	llseek:no_llseek,
	ioctl:rtc_au1xxx_ioctl,
	open:rtc_au1xxx_open,
	release:rtc_au1xxx_release,
};

static struct miscdevice rtc_au1xxx_dev = {
	RTC_MINOR,
	"rtc",
	&rtc_au1xxx_fops
};


static int __init rtc_au1xxx_init(void)
{
	printk(KERN_INFO "Au1XXX Real Time Clock Driver v%s\n", AU1XXX_RTC_VERSION);
	misc_register(&rtc_au1xxx_dev);
	return 0;
}

static void __exit rtc_au1xxx_exit(void)
{
	misc_deregister(&rtc_au1xxx_dev);
}

module_init(rtc_au1xxx_init);
module_exit(rtc_au1xxx_exit);

/*******************************************************************/

static int au1xxx_32khz_startup(void)
		{
	/* Setup the 32kHz oscillator for operation per scheme described above */

	if (!(au_readl(SYS_COUNTER_CNTRL) & SYS_CNTRCTRL_32S)) {
		int timeout = 500000;
		/* Start the 32khz oscillator - NOTE: usually best to do this
		in the board boot up code to give the oscillator plenty of
		time to startup prior to this code */
		au_writel(SYS_CNTRCTRL_E0, SYS_COUNTER_CNTRL);
		/* Wait for it to start...can hang here if there is
		a problem with the 32khz oscillator!!! */
		while (--timeout > 0)
			if (au_readl(SYS_COUNTER_CNTRL) & SYS_CNTRCTRL_32S)
				break; /* oscillator started */
	}

	/* 32khz oscillator present and running */
	if (au_readl(SYS_COUNTER_CNTRL) & SYS_CNTRCTRL_32S) {
		/* Configure RTC counter */
		/* Set trim to 0 for max clock resolution */
		while (au_readl(SYS_COUNTER_CNTRL) & SYS_CNTRCTRL_RTS);
		au_writel(0, SYS_RTCTRIM);
		while (au_readl(SYS_COUNTER_CNTRL) & SYS_CNTRCTRL_RTS);

		/* start RTC counter at zero */
		while (au_readl(SYS_COUNTER_CNTRL) & SYS_CNTRCTRL_RS);
		au_writel(0, SYS_RTCWRITE);
		while (au_readl(SYS_COUNTER_CNTRL) & SYS_CNTRCTRL_RS);

		/* Configure the TOY counter */
		/* Set trim for 1 second clock resolution */
		while (au_readl(SYS_COUNTER_CNTRL) & SYS_CNTRCTRL_TTS);
		au_writel(CONFIG_AU1000_32KHZ_CLK - 1, SYS_TOYTRIM);
		while (au_readl(SYS_COUNTER_CNTRL) & SYS_CNTRCTRL_TTS);

		/* the TOY counter is not written since it contains date/time */

		return 1;
		}
	return 0;
	}

#else

static void setup_cp0_interrupt(void)
{
	/* Setup CP0 Count/Compare interrupt */
	r4k_cur = (read_c0_count() + r4k_offset);
	write_c0_compare(r4k_cur);
}

#endif /* CONFIG_SOC_AU1X00_32KHZ */

void wakeup_kernel_timer(void)
{
#ifdef CONFIG_SOC_AU1X00_32KHZ_TIMER
	au1xxx_32khz_startup();
	setup_rtcm1_interrupt();
#else
	setup_cp0_interrupt();
#endif
}

/* This is to set the timer upon resuming from sleep */
void wakeup_toym2_set(int ticks)
{
#ifdef CONFIG_SOC_AU1X00_32KHZ
	/* sets timeout from sleep mode */
	u32 pc0 = au_readl(SYS_TOYREAD);
	au_writel(pc0 + ticks, SYS_TOYMATCH2);
#endif
}

static inline void ack_r4ktimer(unsigned long newval)
{
	write_c0_compare(newval);
}

unsigned long wtimer;
void mips_timer_interrupt(struct pt_regs *regs)
{
	int irq = 63;
	unsigned long count;

	extern void do_softirq(void);

	irq_enter();

	kstat_this_cpu.irqs[irq]++;
#ifndef CONFIG_SMP
	update_process_times(user_mode(regs));
#endif

	if (r4k_offset == 0)
		goto null;

	do {
		count = read_c0_count();
		timerhi += (count < timerlo);   /* Wrap around */
		timerlo = count;

		do_timer(regs);
		r4k_cur += r4k_offset;
		ack_r4ktimer(r4k_cur);

	} while (((unsigned long)read_c0_count()
	         - r4k_cur) < 0x7fffffff);

	irq_exit();
	return;

null:
	irq_exit();
	ack_r4ktimer(0);
}


/*
 * We read the real processor speed from the PLL.  This is important
 * because it is more accurate than computing it from the 32KHz
 * counter, if it exists.  If we don't have an accurate processor
 * speed, all of the peripherals that derive their clocks based on
 * this advertised speed will introduce error and sometimes not work
 * properly.  This function is futher convoluted to still allow configurations
 * to do that in case they have really, really old silicon with a
 * write-only PLL register, that we need the 32KHz when power management
 * "wait" is enabled, and we need to detect if the 32KHz isn't present
 * but requested......got it? :-)		-- Dan
 */
unsigned long cal_r4koff(void)
{
	u32 cpu_freq, flags;

	spin_lock_irqsave(&time_lock, flags);

#ifdef CONFIG_SOC_AU1000
	/* On early Au1000s, sys_cpupll was write-only. Since these
	silicon versions of Au1000 are not sold by AMD, we don't bend
	over backwards trying to determine the frequency */
#ifndef CONFIG_SOC_AU1000_FREQUENCY
#define CONFIG_SOC_AU1000_FREQUENCY 396000000
#endif
	if ((read_c0_prid() == 0x00030100) || /* Au1000 DA */
		(read_c0_prid() == 0x00030101) || /* Au1000 HA */
		(read_c0_prid() == 0x00030102))   /* Au1000 HB */
		cpu_freq = CONFIG_SOC_AU1000_FREQUENCY;
	else
#endif
		cpu_freq = (au_readl(SYS_CPUPLL) & 0x0000003f) * CONFIG_AU1000_SRC_CLK;

	mips_hpt_frequency = cpu_freq; /* On Alchemy, Count is 1:1 CPU */
	/* Equation: Baudrate = CPU / (SD * 2 * CLKDIV * 16) */
	set_au1x00_uart_baud_base(cpu_freq / (2 * ((int)(au_readl(SYS_POWERCTRL)&0x03) + 2) * 16));
 	set_au1x00_speed(cpu_freq);
 	set_au1x00_lcd_clock(); /* program the LCD clock */

	spin_unlock_irqrestore(&time_lock, flags);

	return (cpu_freq / HZ);
}

void au1xxx_timer_setup(struct irqaction *irq)
{
	extern void au1k_wait(void);

	/*	This code called at the end of time_init() in arch/mips/kernel/time.c.
	The Cp0 Counter has already been chosen as the timer source, here we
	can override the choices made by time_init(). */

#ifdef CONFIG_SOC_AU1X00_32KHZ_TIMER
	/* Setup the 32kHz oscillator as the kernel timer instead */
	if (au1xxx_32khz_startup()) {
		extern void hook_rtcm1_interrupt(void);

		/* it's too early to call request_irq(), but need to hook it anyway */
		setup_rtcm1_interrupt();
		hook_rtcm1_interrupt();

		/* use 32khz flavor of do_fast_gettimeoffset() */
		do_gettimeoffset = do_fast_rtcm1_gettimeoffset;

		/* short-circuit CP0 Count timer handler */
		r4k_offset = 0;

		/* allow WAIT instruction in cpu-probe.c */
		au1k_wait_ptr = au1k_wait;
	}
	else {
		printk("WARNING: no 32KHz clock found...reverting to CP0 timer\n");
	}
#else
	/* Setup first CP0 timer interrupt */
	setup_cp0_interrupt();
#endif /* CONFIG_SOC_AU1X00_32KHZ_TIMER */

}

void __init au1xxx_time_init(void)
{
	/* This called at the start of time_init() in arch/mips/kernel/time.c */
	/* An RTC for use by time_init()/kernel is supposed to be setup here */

#ifdef CONFIG_SOC_AU1X00_32KHZ
	if (au1xxx_32khz_startup()) {
		/* set the kernel function pointers */
		rtc_get_time = rtc_au1xxx_get_time;
		rtc_set_time = rtc_au1xxx_set_time;
	}
#endif /* CONFIG_SOC_AU1X00_32KHZ */

	/* Determine CP0 timer values */
	printk("calculating r4koff... ");
	r4k_offset = cal_r4koff();
	printk("%08x(%d)\n", r4k_offset, (int) r4k_offset);
}
