/*
 * Header for Linux Driver for AMD 5536 UDC 
 * high/full speed USB device controller
 */

/*
 * Copyright (C) 2005 AMD (http://www.amd.com)
 * Author: Thomas Dahlmann
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef AMD5536UDC_H
#define AMD5536UDC_H

/*****************************************************************************
*  Constants
*****************************************************************************/

/* Driver  constants -------------------------------------------------------*/
#define UDC_DRIVER_NAME "amd5536udc"

/* PCI constants -----------------------------------------------------------*/
#define UDC_PCI_VENDOR_ID 0x1022
#define UDC_PCI_DEVICE_ID 0x2096

/* Platform specific -------------------------------------------------------*/
#if defined(CONFIG_MIPS)
#define UDC_PCIPOOL_ALIGN       32 
#define UDC_PCIPOOL_CROSS       4096
#define UDC_PCIPOOL_GFP_STP     (GFP_ATOMIC | GFP_DMA)
#else
#define UDC_PCIPOOL_ALIGN       0 
#define UDC_PCIPOOL_CROSS       0 
#define UDC_PCIPOOL_GFP_STP     (GFP_KERNEL)
#endif

/* temp define for AU1200, will live in au1000.h normally */
#ifndef USB_UDC_BASE
#define USB_UDC_BASE              0x14022000
#define USB_UDC_LEN               0x2000
#define USB_MSR_BASE              0xB4020000
#define USB_MSR_MCFG              4
#define USBMSRMCFG_DMEMEN         4
#define USBMSRMCFG_DBMEN          5
#define USBMSRMCFG_UDCCLKEN       18
#define USBMSRMCFG_PHYPLLEN       19
#define AU1200_USB_INT            29
#endif
#ifndef USBMSRMCFG_UCAM           
#define USBMSRMCFG_UCAM           7
#endif
#define USBMSRMCFG_DEFAULT        0x00d02000

/* Special optimization for certain gadgets ------------------------------- */ 
/* use hardware NAK if IN queue empty, this can be used to 
* avoid interrupt flood as for the ethernet gadget where host side
* sends IN tokens permamently */
#define UDC_DISABLE_IRQ_IF_EMPTY_IN_QUEUE
/*#undef UDC_DISABLE_IRQ_IF_EMPTY_IN_QUEUE*/                                       

/* IP core defect/bug/case workarounds ---------------------------------------- */
/* Hiddensee A0 workarounds */
/* Description: "Last descriptor loop */
#define UDC_IPDEFECT_9000004946_WORKAROUND
/* Description: "Resume from STALL/CLEAR_FEATURE HALT */
/* STALL not required so far by any gadget, so disabled because SOF int. is used
   in workaround which drops performance */
#undef  UDC_IPCASE_8000009363_WORKAROUND
/* Description: "DMA machine not restartable" */
#define UDC_IPCASE_8000018724_WORKAROUND
/* Description: "UDC not accessible when phy is suspended */
#define UDC_IPBUG_3943_WORKAROUND
/* Description: "USB device needs session-valid status information */
#define UDC_IPBUG_3950_WORKAROUND
/* Description: "Clearing endpoint NAK bits not possible if RxFIFO
   not empty (PCS#1503) */
#define UDC_IPBUG_3958_WORKAROUND
#ifdef UDC_IPBUG_3958_WORKAROUND
/* option 1: flush rxfifo on USB reset */
#undef UDC_IPBUG_3958_WORKAROUND_RXFIFO_FLUSH_ON_USBRESET
/* option 2: soft reset on USB reset */
#ifndef UDC_IPBUG_3958_WORKAROUND_RXFIFO_FLUSH_ON_USBRESET
#define UDC_IPBUG_3958_WORKAROUND_SOFT_RESET_ON_USBRESET
#endif
#endif
/* UDC data errors during loop test (DMA) 
   - found only with Travis but used for Hiddensee too, bug entry
     is in Travis bugzilla
   - the workaround applies only for small packets */
#define UDC_IPBUG_2253_WORKAROUND
#define UDC_SMALL_PACKET        32

/* Disable workarounds which don't apply to Hiddensee B0/ Au1200 A1 */
/* And enable new features */
 
/* Hiddensee A0 flag */
#define UDC_HSA0
/* Hiddensee B0 flag */
/* #define UDC_HSB0 */

/* Hiddensee chip rev. */
#define UDC_HSA0_REV 1
#define UDC_HSB0_REV 2

/* consistency */
#ifdef UDC_HSB0
#undef UDC_HSA0
#endif

/* Au1200 rev. */
#if defined(CONFIG_MIPS)
#undef UDC_HSB0
#undef UDC_HSA0
/* revision value */
#define UDC_AUAB_REV 0
#define UDC_AUAC_REV 1
#define UDC_AUA0 0
/* Au1200 AC flag */
//#define UDC_AUA1 1
#endif

/* disable workarounds for newer chip revisions */
#ifdef UDC_HSB0
#undef UDC_IPBUG_3943_WORKAROUND
#undef UDC_IPBUG_3950_WORKAROUND
#define UDC_DMARST_AVAIL
#endif
#ifdef UDC_AUA1
#undef UDC_IPBUG_3943_WORKAROUND
#endif

/* SETUP usb commands 
*  needed, because some SETUP's are handled in hw, but must be passed to 
*  gadget driver above -----------------------------------------------------*/
/* SET_CONFIG */
#define UDC_SETCONFIG_DWORD0                    0x00000900
#define UDC_SETCONFIG_DWORD0_VALUE_MASK         0xffff0000 
#define UDC_SETCONFIG_DWORD0_VALUE_OFS          16 

#define UDC_SETCONFIG_DWORD1                    0x00000000

/* SET_INTERFACE */
#define UDC_SETINTF_DWORD0                      0x00000b00
#define UDC_SETINTF_DWORD0_ALT_MASK             0xffff0000 
#define UDC_SETINTF_DWORD0_ALT_OFS              16

#define UDC_SETINTF_DWORD1                      0x00000000
#define UDC_SETINTF_DWORD1_INTF_MASK            0x0000ffff 
#define UDC_SETINTF_DWORD1_INTF_OFS             0 

/* Global CSR's -------------------------------------------------------------*/
/* UDC CSR's */
#define UDC_CSR_ADDR                            0x500 

/* EP NE bits */
/* EP number */
#define UDC_CSR_NE_NUM_MASK                     0x0000000f 
#define UDC_CSR_NE_NUM_OFS                      0 
/* EP direction */
#define UDC_CSR_NE_DIR_MASK                     0x00000010 
#define UDC_CSR_NE_DIR_OFS                      4 
/* EP type */
#define UDC_CSR_NE_TYPE_MASK                    0x00000060
#define UDC_CSR_NE_TYPE_OFS                     5
/* EP config number */
#define UDC_CSR_NE_CFG_MASK                     0x00000780
#define UDC_CSR_NE_CFG_OFS                      7
/* EP interface number */
#define UDC_CSR_NE_INTF_MASK                    0x00007800
#define UDC_CSR_NE_INTF_OFS                     11 
/* EP alt setting */
#define UDC_CSR_NE_ALT_MASK                     0x00078000
#define UDC_CSR_NE_ALT_OFS                      15
 
/* max pkt */
#define UDC_CSR_NE_MAX_PKT_MASK                 0x3ff80000 
#define UDC_CSR_NE_MAX_PKT_OFS                  19 

/* Device Config Register ---------------------------------------------------*/
#define UDC_DEVCFG_ADDR                         0x400 

#define UDC_DEVCFG_SOFTRESET                    31
#define UDC_DEVCFG_HNPSFEN                      30
#define UDC_DEVCFG_DMARST                       29
#define UDC_DEVCFG_SET_DESC                     18
#define UDC_DEVCFG_CSR_PRG                      17
#define UDC_DEVCFG_STATUS                       7
#define UDC_DEVCFG_DIR                          6
#define UDC_DEVCFG_PI                           5
#define UDC_DEVCFG_SS                           4
#define UDC_DEVCFG_SP                           3
#define UDC_DEVCFG_RWKP                         2

#define UDC_DEVCFG_SPD_MASK                     0x3
#define UDC_DEVCFG_SPD_OFS                      0
#define UDC_DEVCFG_SPD_HS                       0x0
#define UDC_DEVCFG_SPD_FS                       0x1
#define UDC_DEVCFG_SPD_LS                       0x2
/*#define UDC_DEVCFG_SPD_FS                     0x3*/


/* Device Control Register --------------------------------------------------*/
#define UDC_DEVCTL_ADDR                         0x404 

#define UDC_DEVCTL_THLEN_MASK                   0xff000000 
#define UDC_DEVCTL_THLEN_OFS                    24 

#define UDC_DEVCTL_BRLEN_MASK                   0x00ff0000 
#define UDC_DEVCTL_BRLEN_OFS                    16 

#define UDC_DEVCTL_CSR_DONE                     13
#define UDC_DEVCTL_DEVNAK                       12
#define UDC_DEVCTL_SD                           10
#define UDC_DEVCTL_MODE                         9 
#define UDC_DEVCTL_BREN                         8
#define UDC_DEVCTL_THE                          7
#define UDC_DEVCTL_BF                           6
#define UDC_DEVCTL_BE                           5
#define UDC_DEVCTL_DU                           4
#define UDC_DEVCTL_TDE                          3
#define UDC_DEVCTL_RDE                          2
#define UDC_DEVCTL_RES                          0


/* Device Status Register ---------------------------------------------------*/
#define UDC_DEVSTS_ADDR                         0x408

#define UDC_DEVSTS_TS_MASK                      0xfffc0000
#define UDC_DEVSTS_TS_OFS                       18

#define UDC_DEVSTS_SESSVLD                      17
#define UDC_DEVSTS_PHY_ERROR                    16
#define UDC_DEVSTS_RXFIFO_EMPTY                 15

#define UDC_DEVSTS_ENUM_SPEED_MASK              0x00006000
#define UDC_DEVSTS_ENUM_SPEED_OFS               13
#define UDC_DEVSTS_ENUM_SPEED_FULL              1
#define UDC_DEVSTS_ENUM_SPEED_HIGH              0

#define UDC_DEVSTS_SUSP                         12

#define UDC_DEVSTS_ALT_MASK                     0x00000f00
#define UDC_DEVSTS_ALT_OFS                      8

#define UDC_DEVSTS_INTF_MASK                    0x000000f0
#define UDC_DEVSTS_INTF_OFS                     4

#define UDC_DEVSTS_CFG_MASK                     0x0000000f
#define UDC_DEVSTS_CFG_OFS                      0


/* Device Interrupt Register ------------------------------------------------*/
#define UDC_DEVINT_ADDR                         0x40c

#define UDC_DEVINT_SVC                          7
#define UDC_DEVINT_ENUM                         6
#define UDC_DEVINT_SOF                          5
#define UDC_DEVINT_US                           4
#define UDC_DEVINT_UR                           3
#define UDC_DEVINT_ES                           2
#define UDC_DEVINT_SI                           1
#define UDC_DEVINT_SC                           0

/* Device Interrupt Mask Register -------------------------------------------*/
#define UDC_DEVINT_MSK_ADDR                     0x410

#define UDC_DEVINT_MSK                          0x7f

/* Endpoint Interrupt Register ----------------------------------------------*/
#define UDC_EPINT_ADDR                          0x414           

#define UDC_EPINT_OUT_MASK                      0xffff0000           
#define UDC_EPINT_OUT_OFS                       16           
#define UDC_EPINT_IN_MASK                       0x0000ffff           
#define UDC_EPINT_IN_OFS                        0 

#define UDC_EPINT_IN_EP0                        0 
#define UDC_EPINT_IN_EP1                        1 
#define UDC_EPINT_IN_EP2                        2 
#define UDC_EPINT_IN_EP3                        3 
#define UDC_EPINT_OUT_EP0                       16 
#define UDC_EPINT_OUT_EP1                       17
#define UDC_EPINT_OUT_EP2                       18 
#define UDC_EPINT_OUT_EP3                       19 

#define UDC_EPINT_EP0_ENABLE_MSK                0x001e001e    

/* Endpoint Interrupt Mask Register -----------------------------------------*/
#define UDC_EPINT_MSK_ADDR                      0x418           

#define UDC_EPINT_OUT_MSK_MASK                  0xffff0000           
#define UDC_EPINT_OUT_MSK_OFS                   16           
#define UDC_EPINT_IN_MSK_MASK                   0x0000ffff           
#define UDC_EPINT_IN_MSK_OFS                    0 

#define UDC_EPINT_MSK_DISABLE_ALL               0xffffffff
/* mask non-EP0 endpoints */
#define UDC_EPDATAINT_MSK_DISABLE               0xfffefffe
/* mask all dev interrupts */
#define UDC_DEV_MSK_DISABLE                     0x7f

/* Endpoint-specific CSR's --------------------------------------------------*/
/* Endpoint Control Registers -----------------------------------------------*/
#define UDC_EPREGS_ADDR                         0x0
#define UDC_EPIN_REGS_ADDR                      0x0
#define UDC_EPOUT_REGS_ADDR                     0x200

#define UDC_EPCTL_ADDR                          0x0

#define UDC_EPCTL_RRDY                          9
#define UDC_EPCTL_CNAK                          8
#define UDC_EPCTL_SNAK                          7
#define UDC_EPCTL_NAK                           6

#define UDC_EPCTL_ET_MASK                       0x00000030
#define UDC_EPCTL_ET_OFS                        4
#define UDC_EPCTL_ET_CONTROL                    0
#define UDC_EPCTL_ET_ISO                        1 
#define UDC_EPCTL_ET_BULK                       2
#define UDC_EPCTL_ET_INTERRUPT                  3

#define UDC_EPCTL_P                             3
#define UDC_EPCTL_SN                            2
#define UDC_EPCTL_F                             1
#define UDC_EPCTL_S                             0

/* Endpoint Status Registers ------------------------------------------------*/
#define UDC_EPSTS_ADDR                          0x4

#define UDC_EPSTS_RX_PKT_SIZE_MASK              0x007ff800
#define UDC_EPSTS_RX_PKT_SIZE_OFS               11

#define UDC_EPSTS_TDC                           10
#define UDC_EPSTS_HE                            9
#define UDC_EPSTS_BNA                           7
#define UDC_EPSTS_IN                            6

#define UDC_EPSTS_OUT_MASK                      0x00000030
#define UDC_EPSTS_OUT_OFS                       4 
#define UDC_EPSTS_OUT_DATA                      1
#define UDC_EPSTS_OUT_DATA_CLEAR                0x10
#define UDC_EPSTS_OUT_SETUP                     2
#define UDC_EPSTS_OUT_SETUP_CLEAR               0x20

/* Endpoint Buffer Size IN/ Receive Packet Frame Number OUT  Registers ------*/
#define UDC_EPIN_BUFF_SIZE_ADDR                 0x8
#define UDC_EPOUT_FRAME_NUMBER_ADDR             0x8

#define UDC_EPIN_BUFF_SIZE_MASK                 0x0000ffff
#define UDC_EPIN_BUFF_SIZE_OFS                  0
/*  EP0in txfifo = 128 bytes*/ 
#define UDC_EPIN0_BUFF_SIZE                     32
/*  EP0in fullspeed txfifo = 128 bytes*/ 
#define UDC_FS_EPIN0_BUFF_SIZE                  32

/* fifo size mult = fifo size / max packet */
#define UDC_EPIN_BUFF_SIZE_MULT                 2

/* EPin data fifo size = 1024 bytes DOUBLE BUFFERING */
#define UDC_EPIN_BUFF_SIZE                      256 
/* EPin small INT data fifo size = 128 bytes */
#define UDC_EPIN_SMALLINT_BUFF_SIZE             32 

/* EPin fullspeed data fifo size = 128 bytes DOUBLE BUFFERING */
#define UDC_FS_EPIN_BUFF_SIZE                   32 

#define UDC_EPOUT_FRAME_NUMBER_MASK             0x0000ffff
#define UDC_EPOUT_FRAME_NUMBER_OFS              0 

/* Endpoint Buffer Size OUT/Max Packet Size Registers -----------------------*/
#define UDC_EPOUT_BUFF_SIZE_ADDR                0x0c
#define UDC_EP_MAX_PKT_SIZE_ADDR                0x0c

#define UDC_EPOUT_BUFF_SIZE_MASK                0xffff0000
#define UDC_EPOUT_BUFF_SIZE_OFS                 16 
#define UDC_EP_MAX_PKT_SIZE_MASK                0x0000ffff
#define UDC_EP_MAX_PKT_SIZE_OFS                 0
/* EP0in max packet size = 64 bytes */
#define UDC_EP0IN_MAX_PKT_SIZE                  64
/* EP0out max packet size = 64 bytes */
#define UDC_EP0OUT_MAX_PKT_SIZE                 64
/* EP0in fullspeed max packet size = 64 bytes */
#define UDC_FS_EP0IN_MAX_PKT_SIZE               64   
/* EP0out fullspeed max packet size = 64 bytes */
#define UDC_FS_EP0OUT_MAX_PKT_SIZE              64

/* Endpoint dma descriptors ------------------------------------------------*/
/* Setup data */
/* Status dword */
#define UDC_DMA_STP_STS_CFG_MASK                0x0fff0000
#define UDC_DMA_STP_STS_CFG_OFS                 16
#define UDC_DMA_STP_STS_CFG_ALT_MASK            0x000f0000     
#define UDC_DMA_STP_STS_CFG_ALT_OFS             16     
#define UDC_DMA_STP_STS_CFG_INTF_MASK           0x00f00000     
#define UDC_DMA_STP_STS_CFG_INTF_OFS            20     
#define UDC_DMA_STP_STS_CFG_NUM_MASK            0x0f000000     
#define UDC_DMA_STP_STS_CFG_NUM_OFS             24     
#define UDC_DMA_STP_STS_RX_MASK                 0x30000000
#define UDC_DMA_STP_STS_RX_OFS                  28 
#define UDC_DMA_STP_STS_BS_MASK                 0xc0000000
#define UDC_DMA_STP_STS_BS_OFS                  30 
#define UDC_DMA_STP_STS_BS_HOST_READY           0 
#define UDC_DMA_STP_STS_BS_DMA_BUSY             1 
#define UDC_DMA_STP_STS_BS_DMA_DONE             2 
#define UDC_DMA_STP_STS_BS_HOST_BUSY            3 
/* IN data */
/* Status dword */
#define UDC_DMA_IN_STS_TXBYTES_MASK            0x0000ffff
#define UDC_DMA_IN_STS_TXBYTES_OFS             0
#define UDC_DMA_IN_STS_FRAMENUM_MASK           0x07ff0000
#define UDC_DMA_IN_STS_FRAMENUM_OFS            0
#define UDC_DMA_IN_STS_L                       27
#define UDC_DMA_IN_STS_TX_MASK                 0x30000000
#define UDC_DMA_IN_STS_TX_OFS                  28 
#define UDC_DMA_IN_STS_BS_MASK                 0xc0000000
#define UDC_DMA_IN_STS_BS_OFS                  30 
#define UDC_DMA_IN_STS_BS_HOST_READY           0 
#define UDC_DMA_IN_STS_BS_DMA_BUSY             1 
#define UDC_DMA_IN_STS_BS_DMA_DONE             2 
#define UDC_DMA_IN_STS_BS_HOST_BUSY            3 
/* OUT data */
/* Status dword */
#define UDC_DMA_OUT_STS_RXBYTES_MASK            0x0000ffff
#define UDC_DMA_OUT_STS_RXBYTES_OFS             0
#define UDC_DMA_OUT_STS_FRAMENUM_MASK           0x07ff0000
#define UDC_DMA_OUT_STS_FRAMENUM_OFS            0
#define UDC_DMA_OUT_STS_L                       27
#define UDC_DMA_OUT_STS_RX_MASK                 0x30000000
#define UDC_DMA_OUT_STS_RX_OFS                  28 
#define UDC_DMA_OUT_STS_BS_MASK                 0xc0000000
#define UDC_DMA_OUT_STS_BS_OFS                  30 
#define UDC_DMA_OUT_STS_BS_HOST_READY           0 
#define UDC_DMA_OUT_STS_BS_DMA_BUSY             1 
#define UDC_DMA_OUT_STS_BS_DMA_DONE             2 
#define UDC_DMA_OUT_STS_BS_HOST_BUSY            3 
/* other DMA constants */
/* max dma packet */
#define UDC_DMA_MAXPACKET                       65536 
/* DMA buffer len for temp request, should be the same as the upper
layer gadget is using => TODO replace follwing constant */
#define UDC_DMA_TEMP_BUFFER_LEN                16384
/* invalid DMA pointer */
#define        DMA_ADDR_INVALID        (~(dma_addr_t)0)

/* other Endpoint register addresses and values-----------------------------*/
#define UDC_EP_SUBPTR_ADDR                      0x10
#define UDC_EP_DESPTR_ADDR                      0x14
#define UDC_EP_WRITE_CONFIRM_ADDR               0x1c

/* EP number as layouted in AHB space */
#define UDC_EP_NUM                              32
#define UDC_EPIN_NUM                            16
#define UDC_EPIN_NUM_USED                       5
#define UDC_EPOUT_NUM                           16
/* EP number of EP's really used = EP0 + 8 data EP's */
#define UDC_USED_EP_NUM                         9
/* UDC CSR regs are aligned but AHB regs not - offset for OUT EP's */
#define UDC_CSR_EP_OUT_IX_OFS                   12

#define UDC_EP0OUT_IX                           16
#define UDC_EP0IN_IX                            0

/* max packet */
#define UDC_HS_BULK_MAXPKT                      512
//DISABLEDamd #define UDC_EP0_MAXPKT                          64
/* max packet fullspeed */
//DISABLEDamd #define UDC_FS_EP0_MAXPKT                       8

/* Rx fifo address and size = 1k -------------------------------------------*/
#define UDC_RXFIFO_ADDR                         0x800
#define UDC_RXFIFO_SIZE                         0x400

/* Tx fifo address and size = 1.5k -----------------------------------------*/
#define UDC_TXFIFO_ADDR                         0xc00
#define UDC_TXFIFO_SIZE                         0x600

/* default data endpoints --------------------------------------------------*/
#define UDC_EPIN_STATUS_IX                      1                            
#define UDC_EPIN_IX                             2                            
#define UDC_EPOUT_IX                            18                            

/* general constants -------------------------------------------------------*/ 
#define UDC_DWORD_BYTES                         4
#define UDC_BITS_PER_BYTE_SHIFT                 3
#define UDC_BYTE_MASK                           0xff
#define UDC_BITS_PER_BYTE                       8

/* char device constants ---------------------------------------------------*/
/* names */
#ifdef DEBUG
#ifdef UDC_DRIVER_NAME
#define UDC_DEVICE_NAME UDC_DRIVER_NAME
#else
#define UDC_DEVICE_NAME "amd5536udc"
#endif
#define UDC_DEVICE_FILE_NAME "amd5536udc_dev"
#define UDC_DEVICE_FILE_INODE "/dev/amd5536udc_dev"
/* major number */
#define UDC_MAJOR_NUM   240
/* ioctl */
#define UDC_IOCTL_READ_EP0OUT_DATA_FROM_FIFO _IOWR(UDC_MAJOR_NUM, 1, int)
#endif

#ifdef __KERNEL__

/* 2.5 and 2.4.older portability changes ... */

#ifndef container_of
#define        container_of        list_entry
#endif

#ifndef likely
#define likely(x) (x)
#define unlikely(x) (x)
#endif

#ifndef BUG_ON
#define BUG_ON(condition) do { if (unlikely((condition)!=0)) BUG(); } while(0)
#endif

#ifndef WARN_ON
#define        WARN_ON(x)        do { } while (0)
#endif

#ifndef        IRQ_NONE
typedef void irqreturn_t;
#define IRQ_NONE
#define IRQ_HANDLED
#define IRQ_RETVAL(x)
#endif

#endif /*__KERNEL__*/

/* FIXME TMP26: look for better dependency than CONFIG_PCI */ 
/* dma desc. allocation macros replace */
#ifdef CONFIG_MIPS
#if   LINUX_VERSION_CODE >= UDC_NEW_GADGET_KERNEL
#undef	pci_pool
#undef  pci_pool_create
#undef	pci_pool_destroy
#undef	pci_pool_alloc
#undef	pci_pool_free
#define	pci_pool dma_pool
/* FIXME TMP26: use "struct device*" if device file system works */
/* #define pci_pool_create(name, pdev, size, align, allocation) \
	dma_pool_create(name, &pdev->dev, size, align, allocation) */
#define pci_pool_create(name, pdev, size, align, allocation) \
		dma_pool_create(name, NULL, size, align, allocation)
#define	pci_pool_destroy(pool) dma_pool_destroy(pool)
#define	pci_pool_alloc(pool, flags, handle) dma_pool_alloc(pool, flags, handle)
#define	pci_pool_free(pool, vaddr, addr) dma_pool_free(pool, vaddr, addr)
#endif
#endif

/*****************************************************************************
*  Types
*****************************************************************************/

/* UDC CSR's */
struct udc_csrs {
        
        /* sca - setup command address */
        u32 sca;

        /* ep ne's */
        u32 ne[UDC_USED_EP_NUM];
} __attribute__ ((packed));

/* AHB subsystem CSR registers */
struct udc_regs {

        /* device configuration */
        u32 cfg;

        /* device control */
        u32 ctl;

        /* device status */
        u32 sts;

        /* device interrupt */
        u32 irqsts;

        /* device interrupt mask */
        u32 irqmsk;
        
        /* endpoint interrupt  */
        u32 ep_irqsts;

        /* endpoint interrupt mask */
        u32 ep_irqmsk;
} __attribute__ ((packed));

/* endpoint specific registers */
struct udc_epregs {

        /* endpoint control */
        u32 ctl;

        /* endpoint status */
        u32 sts;

        /* endpoint buffer size in/ receive packet frame number out  */
        u32 bufin_framenum;

        /* endpoint buffer size out/max packet size */
        u32 bufout_maxpkt;        

        /* endpoint setup buffer pointer */
        u32 subptr; 
        
        /* endpoint data descriptor pointer */
        u32 desptr;

        /* reserverd */
        u32 reserved;

        /* write/read confirmation */
        u32 confirm;

}  __attribute__ ((packed));

#ifdef        __KERNEL__
/* control data DMA desc */
struct udc_stp_dma {
        /* status quadlet */
        u32     dmastat;
        /* reserved */
        u32     _reserved;
        /* first setup word */
        u32     data12;
        /* second setup word */
        u32     data34;
} __attribute__ ((aligned (16)));

/* normal data DMA desc */
struct udc_data_dma {
        /* status quadlet */
        u32     dmastat;
        /* reserved */
        u32     _reserved;
        /* buffer pointer */
        u32     bufptr;
        /* next descriptor pointer */
        u32     next;
} __attribute__ ((aligned (16)));

/* request packet */
struct udc_request {
        struct usb_request                  req;

        /* first dma desc. of chain */
        struct udc_data_dma                 *td_data;
        /* last dma desc. of chain */
        struct udc_data_dma                 *td_data_last;
#ifdef UDC_IPDEFECT_9000004946_WORKAROUND
        /* next pointer of broken chain */
        dma_addr_t                          td_data_last_next; 
#endif
        /* chain length */
        unsigned                            chain_len;

        dma_addr_t                          td_dma;
        struct list_head                    queue;
        unsigned                            mapped : 1,
                                            dma_done : 1,
                                            dma_pend : 1,
                                            valid : 1;
};

/* UDC specific endpoint parameters */
struct udc_ep {
        struct usb_ep                       ep;
        struct udc_epregs                   *regs;
        u32                                 *txfifo;
        u32*                                dma;
        dma_addr_t                          td_dma;
        dma_addr_t                          td_stp_dma;        
        struct udc_stp_dma                  *td_stp;
        struct udc_data_dma                 *td;
        /* temp request */
        struct udc_request                  *req;
        unsigned                            req_used;        
        unsigned                            req_completed;        
        /* NAK state */
        unsigned                            naking;        



        struct udc                          *dev;
        unsigned long                       irqs;

        /* analogous to a host-side qh */
        struct list_head                        queue;
        const struct usb_endpoint_descriptor    *desc;
        unsigned                                num : 8,
                                                fifo_size : 12,
                                                in_fifo_validate : 1,
                                                stopped : 1,
                                                is_in : 1,
                                                is_iso : 1;
};

/* device struct */
struct udc {
        /* each pci device provides one gadget, several endpoints */
        struct usb_gadget               gadget;
        spinlock_t                      lock;
        struct udc_ep                   ep [UDC_EP_NUM];
        struct usb_gadget_driver        *driver;
        struct otg_transceiver          *otg_transceiver;
        struct usb_otg_gadget_extension *otg_driver;
        unsigned                        enabled : 1,
                                        protocol_stall : 1,
                                        got_irq : 1,
                                        region : 1,
                                        otg_supported : 1,
                                        selfpowered : 1,
                                        sys_suspended : 1,
                                        connected;
        u16                             chiprev;

        /* pci state used to access those endpoints */
        struct pci_dev                  *pdev;
        struct udc_csrs                 *csr;
        struct udc_regs                 *regs;
        struct udc_epregs               *epregs;
        u32*                            rxfifo;
        u32*                            txfifo;

        /* DMA desc pools */
        struct pci_pool                 *data_requests;
        struct pci_pool                 *stp_requests;

        /* states */
        u16                             cur_config;
        u16                             cur_intf;
        u16                             cur_alt;
};

/* setup request data */
union udc_setup_data {
        u32                        raw [2];
        struct usb_ctrlrequest     r;
};
#endif /*__KERNEL__*/

/*****************************************************************************
*  Macros
*****************************************************************************/

/***************************************
* SET and GET bitfields in u32 values
* via constants for mask/offset:
* <bit_field_stub_name> is the text between
* UDC_ and _MASK|_OFS of appropiate
* constant
****************************************/
/* set bitfield value in u32 u32Val */
#define AMD_ADDBITS(u32Val, bitfield_val, bitfield_stub_name)\
        (((u32Val) & (((u32) ~((u32) bitfield_stub_name##_MASK))))\
        |(((bitfield_val) << ((u32) bitfield_stub_name##_OFS))\
        & ((u32) bitfield_stub_name##_MASK)))

/* set bitfield value in zero-initialized u32 u32Val */
/* => bitfield bits in u32Val are all zero */
#define AMD_INIT_SETBITS(u32Val, bitfield_val, bitfield_stub_name)\
        ((u32Val)\
        |(((bitfield_val) << ((u32) bitfield_stub_name##_OFS))\
        &((u32) bitfield_stub_name##_MASK)))

/* get bitfield value from u32 u32Val */
#define AMD_GETBITS(u32Val, bitfield_stub_name)\
        ((u32Val & ((u32) bitfield_stub_name##_MASK))\
        >> ((u32) bitfield_stub_name##_OFS))

/* SET and GET bits in u32 values ------------------------------------------*/
#define AMD_BIT(bit_stub_name) (1 << bit_stub_name)
#define AMD_UNMASK_BIT(bit_stub_name) (~AMD_BIT(bit_stub_name))
#define AMD_CLEAR_BIT(bit_stub_name) (~AMD_BIT(bit_stub_name))

/* misc --------------------------------------------------------------------*/
#define        DIR_STRING(bAddress) (((bAddress) & USB_DIR_IN) ? "in" : "out")

/* UDC specific macros -----------------------------------------------------*/
#ifdef UDC_DMARST_AVAIL
#define UDC_DMARST(tmp, dev) \
        DBG("DMA machine reset\n"); \
        tmp = readl(&dev->regs->cfg); \
        writel(tmp | AMD_BIT(UDC_DEVCFG_DMARST), &dev->regs->cfg); \
        writel(tmp, &dev->regs->cfg);
#else
#define UDC_DMARST(tmp, dev) {} 
#endif

/* print macros ------------------------------------------------------------*/

#ifdef VERBOSE
#ifndef DEBUG
#define DEBUG
#endif
#endif

/* Driver name for printing messages, this should be defined by driver */
#ifndef DRIVER_NAME_FOR_PRINT
//#define DRIVER_NAME_FOR_PRINT "amd5536"
#endif

/**
 * \brief
 * Macro for printing information in drivers
 * 
 * This macro is used for printing kernel messages in driver source code.
 * It should be used for printing useful information about states and called
 * functions for normal operation (not for errors and warnings).
 * 
 * \param fmt is format string for printk
 * \param args... are arguments given to printk (number depends on <fmt>)
 * \return code from printk
 */
#define INFO(args...) \
        printk(KERN_INFO DRIVER_NAME_FOR_PRINT ": " args)

/**
 * \brief
 * Macro for printing warnings in drivers
 * 
 * This macro is used for printing kernel messages in driver source code.
 * It should be used for printing warnings.
 * 
 * \param fmt is format string for printk
 * \param args... are arguments given to printk (number depends on <fmt>)
 * \return code from printk
 */
#define WARN(args...) \
        printk(KERN_WARNING DRIVER_NAME_FOR_PRINT " warning: " args)

/**
 * \brief
 * Macro for printing errors in drivers
 * 
 * This macro is used for printing kernel messages in driver source code.
 * It should be used for printing errors.
 * 
 * \param fmt is format string for printk
 * \param args... are arguments given to printk (number depends on <fmt>)
 * \return code from printk
 */
#define ERR(args...) \
        printk(KERN_ERR DRIVER_NAME_FOR_PRINT " error: " args)

/**
 * \brief
 * Macro for printing debug messages in drivers
 * 
 * This macro is used for printing kernel messages in driver source code
 * when DEBUG is defined 
 * It should be used for printing debug messages.
 * 
 * \param fmt is format string for printk
 * \param args... are arguments given to printk (number depends on <fmt>)
 * \return code from printk
 */
#ifdef DEBUG
#define DBG(args...) \
        printk(KERN_DEBUG DRIVER_NAME_FOR_PRINT " debug: " args)
#else

#define DBG(args...) \
        do {} while (0)
#endif

/**
 * \brief
 * Macro for printing verbose debug messages in drivers
 * 
 * This macro is used for printing kernel messages in driver source code
 * when DEBUG and VERBOSE is defined
 * It should be used for printing debug messages.
 * 
 * \param fmt is format string for printk
 * \param args... are arguments given to printk (number depends on <fmt>)
 * \return code from printk
 */
#ifdef VERBOSE
#define VDBG DBG
#else
#define VDBG(args...) \
	do { } while (0)
#endif

/*****************************************************************************
*  Data
*****************************************************************************/
#ifdef __cplusplus
extern "C"
{
#endif



#ifdef __cplusplus
} /* extern "C" */
#endif



/*****************************************************************************
*  Functions
*****************************************************************************/
#ifdef __cplusplus
extern "C"
{
#endif


#ifdef __cplusplus
} /* extern "C" */
#endif



/*****************************************************************************
*  Inline Functions
*****************************************************************************/

/**
 * allow gadget some resting by halting
 * 
 * \param ep           pointer to ep struct
 */
static inline void allow_status (struct udc_ep *ep)
{
        ep->stopped = 1;
}

#ifdef        __KERNEL__
/**
 * stop endpoint NAK 
 * 
 * \param ep           pointer to ep struct
 */
static inline void stop_out_naking (struct udc_ep *ep)
{
        u32        tmp;
        /* clear NAK by writing CNAK */
        tmp = readl(&ep->regs->ctl);
                tmp |= AMD_BIT(UDC_EPCTL_CNAK); 
                writel(tmp, &ep->regs->ctl);
                tmp = readl(&ep->regs->ctl);
}
#endif        /* __KERNEL__ */

#endif /* #ifdef  AMD5536UDC_H */
