/*
 * AMD 5536 USB OTG controller driver
 */

/*
 * Copyright (C) 2005 AMD (http://www.amd.com)
 * Author: Karsten Boge 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef AMD5536UOC_H
#define AMD5536UOC_H


/*****************************************************************************
*  Config options
*****************************************************************************/

#ifndef UDC_HSA0_WORKAROUNDS
#define UDC_HSA0_WORKAROUNDS
#endif

/* #define USE_AU1200_PCI_DUMMY */
#undef USE_AU1200_PCI_DUMMY
#if    LINUX_VERSION_CODE < KERNEL_VERSION(2,5,59)
#ifndef USE_AU1200_PCI_DUMMY
#define USE_AU1200_PCI_DUMMY
#endif
#endif

#ifdef CONFIG_USB_OTG
#undef CONFIG_USB_OTGMUX_HOST
#undef CONFIG_USB_OTGMUX_GADGET
#undef CONFIG_USB_OTGMUX_IDPIN
#endif

#ifdef CONFIG_USB_OTGMUX_IDPIN
#undef CONFIG_USB_OTGMUX_HOST
#ifndef CONFIG_USB_OTGMUX_GADGET
#define CONFIG_USB_OTGMUX_GADGET
#endif
#endif

#ifndef OTG_HSA0_HNP_WA
#define OTG_HSA0_HNP_WA
#endif

#ifdef VERBOSE
#ifndef DEBUG
#define DEBUG
#endif
#endif


/*****************************************************************************
*  Constants
*****************************************************************************/

#define OTG_DRIVER_NAME        "amd5536otg"

#define OTG_CHIPREV            0

#define OTG_FLAGS_ACTIV        (1<<19)   /* full OTG functionality is activ  */

#ifndef CONFIG_SOC_AU1200
#ifdef CONFIG_PCI
#ifndef PCI_VENDOR_ID_AMD
#define PCI_VENDOR_ID_AMD             0x1022
#endif
#ifndef PCI_DEVICE_ID_AMD_CS5536_UOC
#define PCI_DEVICE_ID_AMD_CS5536_UOC  0x2097
#endif
#else
#error "!!! UNKNOWN SYSTEM BUS TYPE !!!"
#endif
#endif

#ifdef CONFIG_SOC_AU1200
#define GPIO_2_BASE                   200
#define GPIO_215                      215
#define SYS_PINFUNC_P0A_GPIO          (2<<17)
#ifdef CONFIG_MIPS_PB1200
#define USB_VBUS_GPIO                 3     /* AU1000_GPIO_3 */
#endif
#ifdef CONFIG_MIPS_DB1200
#define USB_VBUS_GPIO                 GPIO_215
#endif
#if !(defined(CONFIG_MIPS_PB1200) || defined(CONFIG_MIPS_DB1200))
#error "!!! UNKNOWN AU1200 BOARD !!!"
#endif
#endif


/**********************************
*  CS5536UOC Register definitions
**********************************/

/* capabilities */
#define OTG_CAP_APU            (1<<15)   /* automatic pull-up enable         */

/* multiplexer */
#define OTG_MUX_DISABLE_ALL    0         /* not assigned                     */
#define OTG_MUX_ENABLE_UHC     (2<<0)    /* assigned to host                 */
#define OTG_MUX_ENABLE_UDC     (3<<0)    /* assigned to device               */
#define OTG_MUX_PUEN           (1<<2)    /* pull-up enable                   */
#define OTG_MUX_VBUSVLD        (1<<8)    /* VBus valid                       */

/* status */
#define OTG_STS_ID             (1<<0)    /* ID pin status                    */
#define OTG_STS_VBUSVLD        (1<<1)    /* VBus valid                       */
#define OTG_STS_SESSVLD        (1<<2)    /* Session valid                    */
#define OTG_STS_SESSEND        (1<<3)    /* Session end                      */
#define OTG_STS_LST            (3<<4)    /* Line state                       */
#define OTG_STS_LST_J          (1<<4)    /* Line state                       */
#define OTG_STS_LST_K          (2<<4)    /* Line state                       */
#define OTG_STS_PSPD           (3<<6)    /* Port speed                       */
#define OTG_STS_PSPD_LS        (1<<7)    /* Port speed                       */
#define OTG_STS_PSPD_FS        (2<<7)    /* Port speed                       */
#define OTG_STS_FSOE           (1<<8)    /* FS output enable (OHC)           */
#define OTG_STS_PCON           (1<<9)    /* Port connected                   */
#define OTG_STS_PSUS           (1<<10)   /* Port suspended                   */
#define OTG_STS_TMH            (1<<11)   /* Timer halted                     */
#define OTG_STS_HNP_EN         (1<<12)   /* HNP enabled for B-dev            */
#define OTG_STS_HNP_SUPP       (1<<13)   /* A-host supports HNP              */
#define OTG_STS_HNP_ALTSUPP    (1<<14)   /* A-host supports alt. HNP         */
#define OTG_STS_HNPSTS         (OTG_STS_HNP_EN | OTG_STS_HNP_SUPP | \
                                OTG_STS_HNP_ALTSUPP)
#define OTG_STS_OC             (1<<15)   /* over-current                     */
#define OTG_STS_DPR            (1<<16)   /* Downstream port reset            */

/* control */
#define OTG_CTL_DISABLE_ALL    0         /* not assigned                     */
#define OTG_CTL_ENABLE_UHC     (2<<0)    /* assigned to host                 */
#define OTG_CTL_ENABLE_UDC     (3<<0)    /* assigned to device               */
#define OTG_CTL_MUX_MASK       (3<<0)    /* port mux mask                    */
#define OTG_CTL_PPWR           (1<<2)    /* port power switch                */
#define OTG_CTL_PPO            (1<<3)    /* port power override              */
#define OTG_CTL_CHRG           (1<<4)    /* charge VBus                      */
#define OTG_CTL_DISCHRG        (1<<5)    /* discharge VBus                   */
#define OTG_CTL_IDSNSEN        (1<<6)    /* ID sense enable, ID-PU           */
#define OTG_CTL_PADEN          (1<<7)
#define OTG_CTL_PUEN           (1<<8)    /* pull-up enable                   */
#define OTG_CTL_DMPDEN         (1<<9)    /* pull-down enable                 */
#define OTG_CTL_HNPSFEN        (1<<10)   /* HNP SET_FEATURE enable           */
#define OTG_CTL_WPCS_DEAS      (2<<16)   /* deassert port connect            */
#define OTG_CTL_WPCS_ASRT      (3<<16)   /* assert port connect              */
#define OTG_CTL_WPSS_DEAS      (2<<18)   /* deassert port suspend            */
#define OTG_CTL_WPSS_ASRT      (3<<18)   /* assert port suspend              */
/* timer conditions */
#define OTG_CTL_TMR_RLP        (1<<28)   /* timer reload policy              */
#define OTG_CTL_TMR_ALL        (0xf<<24) /* stop timer                       */
#define OTG_CTL_TMR_STOP       0         /* timer disabled                   */
#define OTG_CTL_TMR_UNCOND     (1<<24)   /* count unconditionally            */
#define OTG_CTL_TMR_SE0        (2<<24)   /* count if LSt = FS-SE0            */
#define OTG_CTL_TMR_FSJ        (3<<24)   /* count if LSt = FS-J              */
#define OTG_CTL_TMR_FSK        (4<<24)   /* count if LSt = FS-K              */
#define OTG_CTL_TMR_NOSE0      (5<<24)   /* count if LSt <> FS-SE0           */
#define OTG_CTL_TMR_NORX       (6<<24)   /* count if Rx inactiv              */
#define OTG_CTL_TMR_ID         (7<<24)   /* count if ID = 0                  */

/* interrupts */
#define OTG_INT_GLOBAL         (1<<31)   /* global interrupt enable          */
#define OTG_INT_ENALL          0x7fff    /* enable all                       */
#define OTG_INT_DISALL         0         /* disable all                      */
#define OTG_INT_IDC            (1<<0)    /* ID pin change                    */
#define OTG_INT_VBVC           (1<<1)    /* VBUS valid change                */
#define OTG_INT_SVC            (1<<2)    /* Session valid change             */
#define OTG_INT_SEC            (1<<3)    /* Session end change               */
#define OTG_INT_LSTC           (1<<4)    /* Line state change                */
#define OTG_INT_PSPDC          (1<<5)    /* Port speed change                */
#define OTG_INT_FSOEC          (1<<6)    /* FS/LS OE change                  */
#define OTG_INT_HSDD           (1<<7)    /* HS disconnect detected           */
#define OTG_INT_RXACT          (1<<8)    /* Rx activity detected             */
#define OTG_INT_PCC            (1<<9)    /* Port connect change              */
#define OTG_INT_PSC            (1<<10)   /* Port suspend change              */
#define OTG_INT_TMX            (1<<11)   /* Timer expired                    */
#define OTG_INT_HNPFC          (1<<12)   /* HNP feature change               */
#define OTG_INT_OCD            (1<<13)   /* over current detected            */
#define OTG_INT_DPRC           (1<<14)   /* Downstream port reset change     */

/*** HS-A0 WA: BUG-3885: VB_SESS_VLD value too high                        ***/
/*** HS-A0 WA: BUG-3950: gadget SV notification                            ***/
#ifdef UDC_HSA0_WORKAROUNDS
#define OTG_INT_ADDS   OTG_INT_SVC
#endif


/**********************************
*  OTG state dependend data
**********************************/

/*
 * generic
 */
#define OTG_CTL_DEFAULT                      (OTG_CTL_PADEN | \
                                              OTG_CTL_IDSNSEN)
#define OTG_CTL_HOST_DEFAULT                 (OTG_CTL_DEFAULT | \
                                              OTG_CTL_ENABLE_UHC)
#ifdef UDC_HSA0_WORKAROUNDS
/*** HS-A0 WA: BUG-3885: VB_SESS_VLD value too high                        ***/
/*** HS-A0 WA: BUG-3943: gadget suspend issue                              ***/
#ifdef CONFIG_USB_OTG
#define OTG_CTL_PERIPHERAL_DEFAULT           (OTG_CTL_DEFAULT | \
                                              OTG_CTL_HNPSFEN | \
                                              OTG_CTL_ENABLE_UDC | \
                                              OTG_CTL_PPO | OTG_CTL_PUEN)
#else
#define OTG_CTL_PERIPHERAL_DEFAULT           (OTG_CTL_DEFAULT | \
                                              OTG_CTL_ENABLE_UDC | \
                                              OTG_CTL_PPO | OTG_CTL_PUEN)
#endif
#else
#ifdef CONFIG_USB_OTG
#define OTG_CTL_PERIPHERAL_DEFAULT           (OTG_CTL_DEFAULT | \
                                              OTG_CTL_HNPSFEN | \
                                              OTG_CTL_ENABLE_UDC | OTG_CTL_PPO)
#else
#define OTG_CTL_PERIPHERAL_DEFAULT           (OTG_CTL_DEFAULT | \
                                              OTG_CTL_ENABLE_UDC | OTG_CTL_PPO)
#endif
#endif

#define OTG_INT_DEFAULT                      OTG_INT_IDC

/*
 * OTG_STATE_UNDEFINED
 */
#define OTG_STATE_UNDEFINED_CONTROL          (OTG_CTL_DEFAULT | OTG_CTL_PPO | \
                                              OTG_CTL_TMR_UNCOND)
#define OTG_STATE_UNDEFINED_STATUS           0
#define OTG_STATE_UNDEFINED_STATUS_MASK      0
#define OTG_STATE_UNDEFINED_INTERRUPTS       OTG_INT_TMX

/*
 * OTG_STATE_NO_B_DEVICE_A
 */
#define OTG_STATE_NO_B_DEVICE_A_CONTROL      OTG_CTL_HOST_DEFAULT
#define OTG_STATE_NO_B_DEVICE_A_STATUS       0
#define OTG_STATE_NO_B_DEVICE_A_STATUS_MASK  0
#define OTG_STATE_NO_B_DEVICE_A_INTERRUPTS   OTG_INT_DEFAULT

/*
 * OTG_STATE_NO_B_DEVICE_B
 */
#define OTG_STATE_NO_B_DEVICE_B_CONTROL      (OTG_CTL_DEFAULT | OTG_CTL_PPO)
#define OTG_STATE_NO_B_DEVICE_B_STATUS       0
#define OTG_STATE_NO_B_DEVICE_B_STATUS_MASK  0
#define OTG_STATE_NO_B_DEVICE_B_INTERRUPTS   OTG_INT_DEFAULT

/*
 * OTG_STATE_A_IDLE
 */
#define OTG_STATE_A_IDLE_CONTROL             (OTG_CTL_DEFAULT | OTG_CTL_PPO)
#define OTG_STATE_A_IDLE_STATUS              0
#define OTG_STATE_A_IDLE_STATUS_MASK         0
#define OTG_STATE_A_IDLE_INTERRUPTS          (OTG_INT_DEFAULT | OTG_INT_SVC | \
                                              OTG_INT_LSTC)

/*
 * OTG_STATE_A_IDLE_WAIT_DP
 */
#define OTG_STATE_A_IDLE_WAIT_DP_CONTROL     (OTG_STATE_A_IDLE_CONTROL | \
                                              OTG_CTL_TMR_UNCOND)
#define OTG_STATE_A_IDLE_WAIT_DP_STATUS      0
#define OTG_STATE_A_IDLE_WAIT_DP_STATUS_MASK 0
#define OTG_STATE_A_IDLE_WAIT_DP_INTERRUPTS  (OTG_STATE_A_IDLE_INTERRUPTS | \
                                              OTG_INT_TMX)

/*
 * OTG_STATE_A_IDLE_WAIT_VP
 */
#define OTG_STATE_A_IDLE_WAIT_VP_CONTROL     (OTG_STATE_A_IDLE_CONTROL | \
                                              OTG_CTL_TMR_UNCOND)
#define OTG_STATE_A_IDLE_WAIT_VP_STATUS      0
#define OTG_STATE_A_IDLE_WAIT_VP_STATUS_MASK 0
#define OTG_STATE_A_IDLE_WAIT_VP_INTERRUPTS  (OTG_STATE_A_IDLE_INTERRUPTS | \
                                              OTG_INT_TMX)

/*
 * OTG_STATE_A_IDLE_WAIT_MP
 */
#define OTG_STATE_A_IDLE_WAIT_MP_CONTROL     (OTG_STATE_A_IDLE_CONTROL | \
                                              OTG_CTL_TMR_UNCOND)
#define OTG_STATE_A_IDLE_WAIT_MP_STATUS      0
#define OTG_STATE_A_IDLE_WAIT_MP_STATUS_MASK 0
#define OTG_STATE_A_IDLE_WAIT_MP_INTERRUPTS  (OTG_STATE_A_IDLE_INTERRUPTS | \
                                              OTG_INT_TMX)

/*
 * OTG_STATE_A_IDLE_WAIT_DV
 */
#define OTG_STATE_A_IDLE_WAIT_DV_CONTROL     (OTG_STATE_A_IDLE_CONTROL | \
                                              OTG_CTL_TMR_UNCOND)
#define OTG_STATE_A_IDLE_WAIT_DV_STATUS      0
#define OTG_STATE_A_IDLE_WAIT_DV_STATUS_MASK 0
#define OTG_STATE_A_IDLE_WAIT_DV_INTERRUPTS  (OTG_STATE_A_IDLE_INTERRUPTS | \
                                              OTG_INT_TMX)

/*
 * OTG_STATE_A_WAIT_VRISE
 */
#define OTG_STATE_A_WAIT_VRISE_CONTROL       (OTG_CTL_HOST_DEFAULT | \
                                              OTG_CTL_TMR_UNCOND)
#define OTG_STATE_A_WAIT_VRISE_STATUS        0
#define OTG_STATE_A_WAIT_VRISE_STATUS_MASK   0
#define OTG_STATE_A_WAIT_VRISE_INTERRUPTS    (OTG_INT_DEFAULT | OTG_INT_TMX | \
                                              OTG_INT_VBVC)

/*
 * OTG_STATE_A_WAIT_BCON
 */
#define OTG_STATE_A_WAIT_BCON_CONTROL        OTG_CTL_HOST_DEFAULT
#define OTG_STATE_A_WAIT_BCON_STATUS         0
#define OTG_STATE_A_WAIT_BCON_STATUS_MASK    0
#define OTG_STATE_A_WAIT_BCON_INTERRUPTS     (OTG_INT_DEFAULT | OTG_INT_TMX | \
                                              OTG_INT_VBVC | OTG_INT_PCC)

/*
 * OTG_STATE_A_WAIT_BCON_VB
 */
#define OTG_STATE_A_WAIT_BCON_VB_CONTROL     (OTG_STATE_A_WAIT_BCON_CONTROL | \
                                              OTG_CTL_TMR_UNCOND)
#define OTG_STATE_A_WAIT_BCON_VB_STATUS      0
#define OTG_STATE_A_WAIT_BCON_VB_STATUS_MASK 0
#define OTG_STATE_A_WAIT_BCON_VB_INTERRUPTS  OTG_STATE_A_WAIT_BCON_INTERRUPTS

/*
 * OTG_STATE_A_HOST
 */
#define OTG_STATE_A_HOST_CONTROL             OTG_CTL_HOST_DEFAULT
#define OTG_STATE_A_HOST_STATUS              0
#define OTG_STATE_A_HOST_STATUS_MASK         0
#ifdef CONFIG_USB_OTG
#ifndef VERBOSE
#define OTG_STATE_A_HOST_INTERRUPTS          (OTG_INT_DEFAULT | \
                                              OTG_INT_VBVC | OTG_INT_DPRC | \
                                              OTG_INT_PCC | OTG_INT_PSC)
#else
#define OTG_STATE_A_HOST_INTERRUPTS          (OTG_INT_DEFAULT | \
                                              OTG_INT_VBVC | OTG_INT_DPRC | \
                                              OTG_INT_PCC | OTG_INT_PSC | \
                                              OTG_INT_PSPDC)
                                              /* OTG_INT_LSTC */
#endif
#else
        /* IDPIN mode only                                                   */
#define OTG_STATE_A_HOST_INTERRUPTS          OTG_INT_IDC
#endif

/*
 * OTG_STATE_A_SUSPEND
 */
#define OTG_STATE_A_SUSPEND_CONTROL          (OTG_CTL_HOST_DEFAULT | \
                                              OTG_CTL_TMR_UNCOND)
#define OTG_STATE_A_SUSPEND_STATUS           0
#define OTG_STATE_A_SUSPEND_STATUS_MASK      0
#define OTG_STATE_A_SUSPEND_INTERRUPTS       (OTG_INT_DEFAULT | OTG_INT_TMX | \
                                              OTG_INT_VBVC |  OTG_INT_DPRC | \
                                              OTG_INT_PCC | OTG_INT_PSC)

/*
 * OTG_STATE_A_PERIPHERAL
 */
#define OTG_STATE_A_PERIPHERAL_CONTROL       (OTG_CTL_PERIPHERAL_DEFAULT | \
                                              OTG_CTL_PPWR | OTG_CTL_DMPDEN)
#define OTG_STATE_A_PERIPHERAL_STATUS        0
#define OTG_STATE_A_PERIPHERAL_STATUS_MASK   0
#ifndef VERBOSE
#define OTG_STATE_A_PERIPHERAL_INTERRUPTS    (OTG_INT_DEFAULT | \
                                              OTG_INT_VBVC | OTG_INT_OCD | \
                                              OTG_INT_PCC | OTG_INT_PSC)
#else
#define OTG_STATE_A_PERIPHERAL_INTERRUPTS    (OTG_INT_DEFAULT | \
                                              OTG_INT_VBVC | OTG_INT_OCD | \
                                              OTG_INT_PCC | OTG_INT_PSC | \
                                              OTG_INT_PSPDC)
                                              /* OTG_INT_LSTC */
#endif

/*
 * OTG_STATE_A_VBUS_ERR
 */
#define OTG_STATE_A_VBUS_ERR_CONTROL         (OTG_CTL_HOST_DEFAULT | \
                                              OTG_CTL_PPO | OTG_CTL_DISCHRG)
#define OTG_STATE_A_VBUS_ERR_STATUS          0
#define OTG_STATE_A_VBUS_ERR_STATUS_MASK     0
#define OTG_STATE_A_VBUS_ERR_INTERRUPTS      OTG_INT_DEFAULT

/*
 * OTG_STATE_A_WAIT_VFALL
 */
#define OTG_STATE_A_WAIT_VFALL_CONTROL       (OTG_CTL_HOST_DEFAULT | \
                                              OTG_CTL_PPO)
#define OTG_STATE_A_WAIT_VFALL_STATUS        0
#define OTG_STATE_A_WAIT_VFALL_STATUS_MASK   0
#define OTG_STATE_A_WAIT_VFALL_INTERRUPTS    (OTG_INT_DEFAULT | OTG_INT_SEC)

/*
 * OTG_STATE_A_WAIT_VFALL_DN
 */
#define OTG_STATE_A_WAIT_VFALL_DN_CONTROL    (OTG_STATE_A_WAIT_VFALL_CONTROL | \
                                              OTG_CTL_DISCHRG)
#define OTG_STATE_A_WAIT_VFALL_DN_STATUS      0
#define OTG_STATE_A_WAIT_VFALL_DN_STATUS_MASK 0
#define OTG_STATE_A_WAIT_VFALL_DN_INTERRUPTS  OTG_STATE_A_WAIT_VFALL_INTERRUPTS

/*
 * OTG_STATE_A_WAIT_BDISCON
 */
#define OTG_STATE_A_WAIT_BDISCON_CONTROL     (OTG_CTL_DEFAULT | \
                                              OTG_CTL_PPO | OTG_CTL_PPWR | \
                                              OTG_CTL_TMR_UNCOND)
#define OTG_STATE_A_WAIT_BDISCON_STATUS      0
#define OTG_STATE_A_WAIT_BDISCON_STATUS_MASK 0
#define OTG_STATE_A_WAIT_BDISCON_INTERRUPTS  (OTG_INT_DEFAULT | OTG_INT_TMX | \
                                              OTG_INT_VBVC | OTG_INT_OCD | \
                                              OTG_INT_PSPDC | OTG_INT_LSTC)

/*
 * OTG_STATE_B_IDLE
 */
/*** HS-A0 WA: BUG-3885: VB_SESS_VLD value too high                        ***/
/*** HS-A0 WA: BUG-3943: gadget suspend issue                              ***/
#ifdef UDC_HSA0_WORKAROUNDS
#define OTG_STATE_B_IDLE_CONTROL             (OTG_CTL_PERIPHERAL_DEFAULT & \
                                              ~((u32) (OTG_CTL_PUEN | \
                                                       OTG_CTL_ENABLE_UDC)))
#else
#define OTG_STATE_B_IDLE_CONTROL             OTG_CTL_PERIPHERAL_DEFAULT
#endif
#define OTG_STATE_B_IDLE_STATUS              0
#define OTG_STATE_B_IDLE_STATUS_MASK         0
#ifdef CONFIG_USB_OTG
#define OTG_STATE_B_IDLE_INTERRUPTS          (OTG_INT_DEFAULT | OTG_INT_SVC)
#else
#ifdef CONFIG_USB_OTGMUX_IDPIN
        /* IDPIN mode                                                        */
#define OTG_STATE_B_IDLE_INTERRUPTS          (OTG_INT_IDC | OTG_INT_SVC)
#else
        /* gadget mode                                                       */
#define OTG_STATE_B_IDLE_INTERRUPTS          OTG_INT_SVC
#endif
#endif

/*
 * OTG_STATE_B_PERIPHERAL
 */
#define OTG_STATE_B_PERIPHERAL_CONTROL       (OTG_CTL_PERIPHERAL_DEFAULT | \
                                              OTG_CTL_DMPDEN)
#define OTG_STATE_B_PERIPHERAL_STATUS        0
#define OTG_STATE_B_PERIPHERAL_STATUS_MASK   0
#ifdef CONFIG_USB_OTG
#ifndef VERBOSE
#define OTG_STATE_B_PERIPHERAL_INTERRUPTS    (OTG_INT_DEFAULT | OTG_INT_SVC | \
                                              OTG_INT_PCC | OTG_INT_PSC | \
                                              OTG_INT_HNPFC)
#else
#define OTG_STATE_B_PERIPHERAL_INTERRUPTS    (OTG_INT_DEFAULT | OTG_INT_SVC | \
                                              OTG_INT_PCC | OTG_INT_PSC | \
                                              OTG_INT_HNPFC | OTG_INT_PSPDC)
                                              /* OTG_INT_LSTC */
#endif
#else
#ifdef CONFIG_USB_OTGMUX_IDPIN
        /* IDPIN mode                                                        */
#define OTG_STATE_B_PERIPHERAL_INTERRUPTS    (OTG_INT_IDC | OTG_INT_SVC)
#else
        /* gadget mode                                                       */
#define OTG_STATE_B_PERIPHERAL_INTERRUPTS    OTG_INT_SVC
#endif
#endif

/*
 * OTG_STATE_B_PERIPHERAL_WT
 */
#define OTG_STATE_B_PERIPHERAL_WT_CONTROL    (OTG_STATE_B_PERIPHERAL_CONTROL | \
                                              OTG_CTL_PPO | OTG_CTL_TMR_UNCOND)
#define OTG_STATE_B_PERIPHERAL_WT_STATUS      0
#define OTG_STATE_B_PERIPHERAL_WT_STATUS_MASK 0
#define OTG_STATE_B_PERIPHERAL_WT_INTERRUPTS (OTG_STATE_B_PERIPHERAL_INTERRUPTS \
                                              | OTG_INT_TMX)

/*
 * OTG_STATE_B_PERIPHERAL_DC
 */
#define OTG_STATE_B_PERIPHERAL_DC_CONTROL    (OTG_CTL_HOST_DEFAULT | \
                                              OTG_CTL_PPO | OTG_CTL_DMPDEN | \
                                              OTG_CTL_TMR_UNCOND)
#define OTG_STATE_B_PERIPHERAL_DC_STATUS      0
#define OTG_STATE_B_PERIPHERAL_DC_STATUS_MASK 0
#define OTG_STATE_B_PERIPHERAL_DC_INTERRUPTS (OTG_STATE_B_PERIPHERAL_INTERRUPTS \
                                              | OTG_INT_TMX | OTG_INT_LSTC)

/*
 * OTG_STATE_B_WAIT_ACON
 */
#define OTG_STATE_B_WAIT_ACON_CONTROL        (OTG_CTL_HOST_DEFAULT | \
                                              OTG_CTL_PPO | OTG_CTL_TMR_UNCOND)
#define OTG_STATE_B_WAIT_ACON_STATUS         0
#define OTG_STATE_B_WAIT_ACON_STATUS_MASK    0
#define OTG_STATE_B_WAIT_ACON_INTERRUPTS     (OTG_INT_DEFAULT | OTG_INT_SVC | \
                                              OTG_INT_PCC | OTG_INT_PSC | \
                                              OTG_INT_HNPFC | OTG_INT_TMX)

/*
 * OTG_STATE_B_HOST
 */
#define OTG_STATE_B_HOST_CONTROL             (OTG_CTL_HOST_DEFAULT | \
                                              OTG_CTL_PPO)
#define OTG_STATE_B_HOST_STATUS              0
#define OTG_STATE_B_HOST_STATUS_MASK         0
#ifndef VERBOSE
#define OTG_STATE_B_HOST_INTERRUPTS          (OTG_INT_DEFAULT | OTG_INT_SVC | \
                                              OTG_INT_PCC)
#else
#define OTG_STATE_B_HOST_INTERRUPTS          (OTG_INT_DEFAULT | OTG_INT_SVC | \
                                              OTG_INT_PCC | OTG_INT_SVC | \
                                              OTG_INT_PSPDC)
                                              /* OTG_INT_LSTC */
#endif

/*
 * OTG_STATE_B_HOST_WT
 */
#define OTG_STATE_B_HOST_WT_CONTROL          (OTG_STATE_B_HOST_CONTROL | \
                                              OTG_CTL_TMR_UNCOND)
#define OTG_STATE_B_HOST_WT_STATUS           0
#define OTG_STATE_B_HOST_WT_STATUS_MASK      0
#ifndef VERBOSE
#define OTG_STATE_B_HOST_WT_INTERRUPTS       (OTG_INT_DEFAULT | OTG_INT_SVC | \
                                              OTG_INT_PCC | OTG_INT_TMX)
#else
#define OTG_STATE_B_HOST_WT_INTERRUPTS       (OTG_STATE_B_HOST_INTERRUPTS | \
                                              OTG_INT_TMX)
/*
#define OTG_STATE_B_HOST_WT_INTERRUPTS       (OTG_INT_DEFAULT | OTG_INT_SVC | \
                                              OTG_INT_PCC | OTG_INT_DPRC | \
                                              OTG_INT_TMX | OTG_INT_PSPDC) */
                                              /* OTG_INT_LSTC */
#endif

/*
 * OTG_STATE_B_SRP_INIT
 */
#define OTG_STATE_B_SRP_INIT_CONTROL         OTG_STATE_B_IDLE_CONTROL
#define OTG_STATE_B_SRP_INIT_STATUS          0
#define OTG_STATE_B_SRP_INIT_STATUS_MASK     0
#define OTG_STATE_B_SRP_INIT_INTERRUPTS      OTG_INT_DEFAULT 

/*
 * OTG_STATE_B_SRP_WTSE0
 */
#define OTG_STATE_B_SRP_WAIT_SE0_CONTROL     (OTG_STATE_B_SRP_INIT_CONTROL | \
                                              OTG_CTL_TMR_UNCOND)
#define OTG_STATE_B_SRP_WAIT_SE0_STATUS      0
#define OTG_STATE_B_SRP_WAIT_SE0_STATUS_MASK 0
#define OTG_STATE_B_SRP_WAIT_SE0_INTERRUPTS  (OTG_STATE_B_SRP_INIT_INTERRUPTS \
                                              | OTG_INT_TMX | OTG_INT_LSTC)

/*
 * OTG_STATE_B_SRP_D_PLS
 *
 * note: changing to this state requires an additional call:
 *       set_srp_conditions (dev);
 *       reset_srp_conditions (dev) is required for the next state
 */
/*** HS-A0 WA: BUG-3885: VB_SESS_VLD value too high                        ***/
/*** HS-A0 WA: BUG-3943: gadget suspend issue                              ***/
/* note: set_/reset_srp_conditions is not needed if HS-A0 WAs are active
 */
#ifdef UDC_HSA0_WORKAROUNDS
#define OTG_STATE_B_SRP_D_PULSE_CONTROL      (OTG_CTL_PERIPHERAL_DEFAULT | \
                                              OTG_CTL_PUEN | \
                                              OTG_CTL_TMR_UNCOND)
#else
#define OTG_STATE_B_SRP_D_PULSE_CONTROL      (OTG_STATE_B_SRP_INIT_CONTROL | \
                                              OTG_CTL_PUEN | \
                                              OTG_CTL_TMR_UNCOND)
#endif
#define OTG_STATE_B_SRP_D_PULSE_STATUS       0
#define OTG_STATE_B_SRP_D_PULSE_STATUS_MASK  0
#define OTG_STATE_B_SRP_D_PULSE_INTERRUPTS   (OTG_STATE_B_SRP_INIT_INTERRUPTS \
                                              | OTG_INT_SEC | OTG_INT_TMX)

/*
 * OTG_STATE_B_SRP_V_PLS
 */
#define OTG_STATE_B_SRP_V_PULSE_CONTROL      (OTG_STATE_B_SRP_INIT_CONTROL | \
                                              OTG_CTL_CHRG | \
                                              OTG_CTL_TMR_UNCOND)
#define OTG_STATE_B_SRP_V_PULSE_STATUS       0
#define OTG_STATE_B_SRP_V_PULSE_STATUS_MASK  0
#define OTG_STATE_B_SRP_V_PULSE_INTERRUPTS   (OTG_STATE_B_SRP_INIT_INTERRUPTS \
                                              | OTG_INT_TMX)

/*
 * OTG_STATE_B_SRP_V_DCG
 */
#define OTG_STATE_B_SRP_V_DCHRG_CONTROL      (OTG_STATE_B_SRP_INIT_CONTROL | \
                                              OTG_CTL_DISCHRG | \
                                              OTG_CTL_TMR_UNCOND)
#define OTG_STATE_B_SRP_V_DCHRG_STATUS       0
#define OTG_STATE_B_SRP_V_DCHRG_STATUS_MASK  0
#define OTG_STATE_B_SRP_V_DCHRG_INTERRUPTS   (OTG_STATE_B_SRP_INIT_INTERRUPTS \
                                              | OTG_INT_TMX)

/*
 * OTG_STATE_B_SRP_WTVB
 */
#define OTG_STATE_B_SRP_WAIT_VBUS_CONTROL    (OTG_STATE_B_SRP_INIT_CONTROL | \
                                              OTG_CTL_TMR_UNCOND)
#define OTG_STATE_B_SRP_WAIT_VBUS_STATUS      0
#define OTG_STATE_B_SRP_WAIT_VBUS_STATUS_MASK 0
#define OTG_STATE_B_SRP_WAIT_VBUS_INTERRUPTS (OTG_STATE_B_SRP_INIT_INTERRUPTS \
                                              | OTG_INT_SVC | OTG_INT_TMX)

/*********************************/

/* other */

#define OTG_APP_REQ_ACK        0


/*****************************************************************************
*  Types
*****************************************************************************/


/*****************************************************************************
*  Macros
*****************************************************************************/

/* printing messages */

#define INFO(args...) \
        printk(KERN_INFO DRIVER_NAME_FOR_PRINT ": " args)

#define WARN(args...) \
        printk(KERN_WARNING DRIVER_NAME_FOR_PRINT " warning: " args)

#define ERR(args...) \
        printk(KERN_ERR DRIVER_NAME_FOR_PRINT " error: " args)

#ifdef DEBUG
#define DBG(args...) \
        printk(KERN_DEBUG DRIVER_NAME_FOR_PRINT " debug: " args)
#else
#define DBG(args...) \
        do {} while (0)
#endif

#ifdef VERBOSE
#define VDBG DBG
#else
#define VDBG(args...) \
	do { } while (0)
#endif

/****************************************************************************/

/* this should always return "1" and print something in verbose mode */
#ifdef VERBOSE
#define VDBG_SPC(fmt,args...) \
	(VDBG (fmt, args) ? 1 : 1)
#else
#define VDBG_SPC(fmt,args...) 1
#endif

/* query bit(s) (long: 32-bit access) */
#define IS_BIT_RES(data, code) \
	(!((data) & (code)) ? \
	 (VDBG_SPC ("  OTG HW status: %s is reset\n", #data)) : 0)

#define IS_BIT_SET(data, code) \
	(((data) & (code)) ? \
	 (VDBG_SPC ("  OTG HW status: %s is set\n", #data)) : 0)

/* query SW flag(s) */
#define IS_FLAG_RES(dev, data) \
	(!((data) & (dev)->transceiver.params) ? \
	 (VDBG_SPC ("  OTG SW status: %s is reset\n", #data)) : 0)

#define IS_FLAG_SET(dev, data) \
	(((data) & (dev)->transceiver.params) ? \
	 (VDBG_SPC ("  OTG SW status: %s is set\n", #data)) : 0)

/* query event bit(s) */
#define GOT_EVENT(data, code) \
	(((data) & (code)) ? \
	 (VDBG_SPC ("  OTG event: %s\n", #data)) : 0)

/* set SW flag */
#ifdef VERBOSE
#define SET_FLAG(dev, data) \
	if (!((data) & (dev)->transceiver.params)) \
		DBG ("  OTG SW status change: set flag %s\n", #data); \
	(dev)->transceiver.params |= (data)
#else
#define SET_FLAG(dev, data) \
	(dev)->transceiver.params |= (data)
#endif

/* reset SW flag */
#ifdef VERBOSE
#define RES_FLAG(dev, data) \
	if ((data) & (dev)->transceiver.params) \
		DBG ("  OTG SW status change: reset flag %s\n", #data); \
	(dev)->transceiver.params &= ~((u32) (data))
#else
#define RES_FLAG(dev, data) \
	(dev)->transceiver.params &= ~((u32) (data))
#endif

/* reset event bit */
#define RES_EVENT(data, code) \
	(code) &= ~((u32) (data))
/* NOTE: this is not really needed so far, might be replaced with */
/* #define RES_EVENT(data, code) \                                */
/* 	do {} while (0)                                           */

/* change OTG state */
#ifdef CONFIG_USB_OTG
#define PREPARE_STATE_CHANGE_1ST(dev, new_state) \
	switch ((new_state) & OTG_STATE_MASK) { \
	case OTG_STATE_UNDEFINED: \
		set_undef_state_defaults((dev)); \
		break; \
	case OTG_STATE_A_IDLE: \
		set_a_state_defaults((dev)); \
		break; \
	case OTG_STATE_B_IDLE: \
		set_b_state_defaults((dev)); \
		break; \
	default: \
		break; \
	} \
	do {} while (0)
#else
#define PREPARE_STATE_CHANGE_1ST(dev, new_state) \
	do {} while (0)
#endif

#ifdef CONFIG_SOC_AU1200

#if (USB_VBUS_GPIO < GPIO_2_BASE)
#define AU1000GPIO_SET_BIT(data) \
	au1000gpio_bit_set (data)
#define AU1000GPIO_CLR_BIT(data) \
	au1000gpio_bit_clear (data)
#if    LINUX_VERSION_CODE < KERNEL_VERSION(2,5,59)
#define AU1000GPIO_INIT_BIT(data) \
	au1000gpio_bit_init (data, &init_usb_vbus_switch)
#define AU1000GPIO_TERM_BIT(data) \
	au1000gpio_bit_term (data, &init_usb_vbus_switch)
#else
#define AU1000GPIO_INIT_BIT(data) \
	au1000gpio_bit_init (data)
#define AU1000GPIO_TERM_BIT(data) \
	au1000gpio_bit_term (data)
#endif
#else
#define AU1000GPIO_SET_BIT(data) \
	au1000gpio2_bit_set (data)
#define AU1000GPIO_CLR_BIT(data) \
	au1000gpio2_bit_clear (data)
#define AU1000GPIO_INIT_BIT(data) \
	au1000gpio2_bit_init (data)
#define AU1000GPIO_TERM_BIT(data) \
	au1000gpio2_bit_term (data)
#endif

#define PREPARE_STATE_CHANGE_2ND(new_state_control) \
	if (((new_state_control) & OTG_CTL_PPO) && \
	    (~(new_state_control) & OTG_CTL_PPWR)) \
		AU1000GPIO_SET_BIT(USB_VBUS_GPIO); \
	else \
		AU1000GPIO_CLR_BIT(USB_VBUS_GPIO)
#else
#define PREPARE_STATE_CHANGE_2ND(new_state_control) \
	do {} while (0)
#endif

#define CHANGE_STATE(dev, new_state, pMask) \
	PREPARE_STATE_CHANGE_1ST(dev, new_state); \
	PREPARE_STATE_CHANGE_2ND(new_state##_CONTROL); \
	writel ((new_state##_CONTROL), &(dev)->regs->ctl); \
	*(pMask) = (new_state##_INTERRUPTS); \
	(dev)->transceiver.state = (new_state); \
	DBG ("OTG new state: %s\n", #new_state)

/* verify OTG state */
#ifndef CONFIG_OTG_TEST_MODE

#define CHECK_STATE(dev, act_state, pMask) \
	*(pMask) = (act_state##_INTERRUPTS); \
	(dev)->transceiver.prv_state = (act_state); \
	VDBG ("OTG state: %s\n", #act_state)
#else
#define CHECK_STATE(dev, act_state, pMask) \
	*(pMask) = (act_state##_INTERRUPTS); \
	(dev)->transceiver.prv_state = (act_state); \
	if (((readl (&(dev)->regs->sts) ^ (act_state##_STATUS))) & \
	    act_state##_STATUS_MASK) \
		WARN ("OTG warning: incorrect status\n"); \
	VDBG ("OTG state: %s\n", #act_state)
#endif

/* set timer */
#define SET_OTG_TIMER(dev, val) \
	set_timer ((dev), ((OTG_TMR_##val) * 100))

/* set timer ( <1ms ) */
#define SET_OTG_TIMER_SHORT(dev, val) \
	set_timer ((dev), ((OTG_TMR_##val) / 10))

/* set timer ( >10ms ) */
#define SET_OTG_TIMER_LONG(dev, val) \
	set_timer_long ((dev), ((OTG_TMR_##val) / 10))

#ifdef VERBOSE
#define HS_DISCON_WARNING() \
	if (!(OTG_CTL_ENABLE_UHC ^ \
	      (OTG_CTL_MUX_MASK & readl (&dev->regs->ctl))) && \
	    !(OTG_STS_PSPD & readl (&dev->regs->sts))) \
		DBG ("  OTG warning: disable UHC from HS-mode\n")
#else
#define HS_DISCON_WARNING() \
	do { } while (0)
#endif


/*****************************************************************************
*  Data
*****************************************************************************/
#ifdef __cplusplus
extern "C"
{
#endif

struct otg_regs {
	u32  cap;               /* capabilities */
	u32  mux;               /* mux */
	u32  sts;               /* status */
	u32  ctl;               /* control */
	u32  tmr;               /* timer */
	u32  intr;              /* interrupt request */
	u32  inten;             /* interrupt enable */
} __attribute__ ((packed));


struct otg {
	spinlock_t              lock;
	unsigned                enabled : 1,
	                        got_irq : 1,
	                        region : 1;
	u16                     chiprev;

#ifdef CONFIG_SOC_AU1200
#ifdef USE_AU1200_PCI_DUMMY

	struct pci_dev          *pdev;
#else
	struct platform_device  *pdev;
#endif
#else
	struct pci_dev          *pdev;
#endif
	struct otg_regs         *regs;
	struct otg_transceiver  transceiver;
};
#define otg_transceiver_to_otg(pTransceiver) \
	container_of (otg, struct otg, pTransceiver)
#define otg_to_transceiver(pOtg) \
	&pOtg->transceiver

#ifdef CONFIG_USB_OTG_TEST_INTERFACE
struct usb_otg_test_extension {
	int (*bind) (struct otg **);
	int (*unbind) (struct otg **);
	int (*request) (u32);	/* function call for state change requests */
	u32 (*query) (int);	/* function call to query state */
};
#endif

#ifdef	__KERNEL__

/* 2.5 and 2.4.older portability changes ... */

#ifndef container_of
#define	container_of	list_entry
#endif

#ifndef likely
#define likely(x) (x)
#define unlikely(x) (x)
#endif

#ifndef BUG_ON
#define BUG_ON(condition) do { if (unlikely((condition)!=0)) BUG(); } while(0)
#endif

#ifndef WARN_ON
#define	WARN_ON(x)	do { } while (0)
#endif

#endif	/* __KERNEL__ */

#ifdef __cplusplus
} /* extern "C" */
#endif


/*****************************************************************************
*  Functions
*****************************************************************************/
#ifdef __cplusplus
extern "C"
{
#endif

#if defined(CONFIG_USB_OTG) || defined(CONFIG_USB_OTGMUX_GADGET)
extern int usb_gadget_register_otg (struct otg_transceiver * (
                                    *get_transceiver)(void));
extern int usb_gadget_unregister_otg (void);

#endif
#ifdef CONFIG_USB_OTG
u32 otg_change_state (struct otg *, u32, u32 *);
int otg_hnp_start_enum (struct otg_transceiver *);
#endif

void otg_init_state (struct otg *);
int otg_exit_state (struct otg *);

#ifdef DEBUG
static void print_regs (struct otg *);
#endif  /* DEBUG */

#ifdef CONFIG_USB_OTG_TEST_INTERFACE
int usb_otg_test_register_driver (struct usb_otg_test_extension *);
int usb_otg_test_unregister_driver (void);
#endif

#ifdef __cplusplus
} /* extern "C" */
#endif


/*****************************************************************************
*  Inline Functions
*****************************************************************************/

extern u32 otg_tmr_high_count;
extern struct otg_ctl *otg_ctl;

#ifdef CONFIG_USB_OTG
/**
 * \brief
 * set neutral state information
 *
 * \param  dev   OTG controller info
 *
 * \return void
 */
static inline void set_undef_state_defaults (struct otg *dev)
{
	dev->transceiver.default_a = 0;
	if (dev->transceiver.host) {
		dev->transceiver.host->is_b_host = 0;
	}
#ifdef CONFIG_USB_OTG_HIGHSPEED
	if (dev->transceiver.companion) {
		dev->transceiver.companion->is_b_host = 0;
	}
#endif
	if (dev->transceiver.gadget) {
		dev->transceiver.gadget->is_a_peripheral = 0;
	}
}

/**
 * \brief
 * set A state information
 *
 * \param  dev   OTG controller info
 *
 * \return void
 */
static inline void set_a_state_defaults (struct otg *dev)
{
	dev->transceiver.default_a = 1;
	if (dev->transceiver.host) {
		dev->transceiver.host->is_b_host = 0;
	}
#ifdef CONFIG_USB_OTG_HIGHSPEED
	if (dev->transceiver.companion) {
		dev->transceiver.companion->is_b_host = 0;
	}
#endif
	if (dev->transceiver.gadget) {
		dev->transceiver.gadget->is_a_peripheral = 1;
	}
}

/**
 * \brief
 * set B state information
 *
 * \param  dev   OTG controller info
 *
 * \return void
 */
static inline void set_b_state_defaults (struct otg *dev)
{
	dev->transceiver.default_a = 0;
	if (dev->transceiver.host) {
		dev->transceiver.host->is_b_host = 1;
	}
#ifdef CONFIG_USB_OTG_HIGHSPEED
	if (dev->transceiver.companion) {
		dev->transceiver.companion->is_b_host = 1;
	}
#endif
	if (dev->transceiver.gadget) {
		dev->transceiver.gadget->is_a_peripheral = 0;
	}
}

/**
 * \brief
 * set B state information
 *
 * \param  dev   OTG controller info
 *
 * \return void
 */
static inline void reset_b_hnp_enable (struct otg *dev)
{
	if (dev->transceiver.host) {
		dev->transceiver.host->b_hnp_enable = 0;
	}
#ifdef CONFIG_USB_OTG_HIGHSPEED
	if (dev->transceiver.companion) {
		dev->transceiver.companion->b_hnp_enable = 0;
	}
#endif
	VDBG ("  OTG action: HNP disabled in B-device\n");
}

/**
 * \brief
 * set B state information
 *
 * \param  dev   OTG controller info
 *
 * \return void
 */
static inline int is_b_hnp_enabled (struct otg *dev)
{
	int retVal = 0;

	if (dev->transceiver.host &&
		dev->transceiver.host->b_hnp_enable) {
		VDBG ("  OTG status: HNP is enabled in HS-B-device\n");
		retVal = 1;
	}
#ifdef CONFIG_USB_OTG_HIGHSPEED
	else if (dev->transceiver.companion &&
		dev->transceiver.companion->b_hnp_enable) {
		VDBG ("  OTG status: HNP is enabled in FS-B-device\n");
		retVal = 1;
	}
#endif
#ifdef VERBOSE
	else {
		DBG ("  OTG status: HNP is disabled in B-device\n");
	}
#endif
	return retVal;
}
#endif

#if defined(CONFIG_USB_OTG) || defined(CONFIG_USB_OTGMUX_GADGET)

/**
 * \brief
 * Read the status register
 *
 * \param  dev   OTG controller info
 *
 * \return status
 */
static inline u32 get_status (struct otg *dev)
{
	return (readl (&dev->regs->sts));
}

/**
 * \brief
 * Load and start the timer for an unconditional run
 *
 * \param  dev   OTG controller info
 * \param  val   Value to load
 *
 * \return void
 */
static inline void set_timer (struct otg *dev, u32 val)
{
	otg_tmr_high_count = 0;

	writel ((val), &dev->regs->tmr);
	VDBG ("  OTG action: start timer: %d0 us\n", val);
}

/**
 * \brief
 * Load and start the timer for an unconditional run
 *
 * \param  dev   OTG controller info
 * \param  val   Value to load
 *
 * \return void
 */
static inline void set_timer_long (struct otg *dev, u32 val)
{
	otg_tmr_high_count = val - 1;

	writel (TIMER_PERIOD, &dev->regs->tmr);
	VDBG ("  OTG action: start timer: %d0 ms\n", val);
}

/**
 * \brief
 * Re-start the timer (value already loaded)
 *
 * \param  dev   OTG controller info
 *
 * \return void
 */
static inline void restart_timer (struct otg *dev)
{
	writel ((readl (&dev->regs->ctl) | OTG_CTL_TMR_UNCOND),
	        &dev->regs->ctl);
}

/**
 * \brief
 * Reset the timer while running (value already loaded)
 *
 * \param  dev   OTG controller info
 *
 * \return void
 */
static inline void reset_timer (struct otg *dev)
{
	u32 temp;

	temp = readl (&dev->regs->ctl);
	writel ((temp & ~((u32) OTG_CTL_TMR_ALL)), &dev->regs->ctl);
	writel (temp, &dev->regs->ctl);
	VDBG ("  OTG action: re-start timer\n");
}

/**
 * \brief
 * Prepare the D-pulse
 *
 * \param  dev   OTG controller info
 *
 * \return void
 */
static inline void set_srp_conditions (struct otg *dev)
{
#ifdef UDC_HSA0_WORKAROUNDS
	VDBG ("  OTG action: SRP init: no action needed due to A0 WAs\n");
#else
	writel ((readl (&dev->regs->cap) & ~((u32) OTG_CAP_APU)),
	        &dev->regs->cap);
	VDBG ("  OTG action: set SRP conditions: disable APU\n");
#endif
}

/**
 * \brief
 * Reset conditions after SRP
 *
 * activates the auto-pull-up feature so after SRP the host
 * will detect a device connect after calling this function
 *
 * \param  dev   OTG controller info
 *
 * \return void
 */
static inline void reset_srp_conditions (struct otg *dev)
{
#ifdef UDC_HSA0_WORKAROUNDS
	VDBG ("  OTG action: SRP done: no action needed due to A0 WAs\n");
#else
	writel ((readl (&dev->regs->cap) | OTG_CAP_APU),
	        &dev->regs->cap);
	VDBG ("  OTG action: reset SRP conditions: enable APU\n");
#endif
}

/**
 * \brief
 * enable HNP for both devices
 *
 * \param  dev   OTG controller info
 *
 * \return success
 */
static inline int otg_enable_hnp (struct otg *dev)
{
	int  retVal = 0;

/*	if (!dev->device) {
		ERR ("Tried to send OTG request, device pointer not valid\n");
		retVal = -ENODEV;
	}
	else {
		retVal = usb_otg_enable_b_hnp (dev->device);
		DBG ("  OTG action: enabled HNP in B-device\n");
	}
	if (retVal < 0) {
		RES_FLAG (OTG_A_bHNP_EN);
	}
	else {
		SET_FLAG (OTG_A_bHNP_EN);
	}*/
	return retVal;
}

#endif /* CONFIG_USB_OTG || CONFIG_USB_OTGMUX_GADGET */
#ifdef DEBUG
/**
 * \brief
 * Print OTG controller registers (debug mode only)
 *
 * \param dev    OTG controller info
 *
 * \return void
 */
static inline void print_regs (struct otg *dev)
{
	DBG( "-- UOC registers ---\n");
	DBG( "otg cap   = %08x\n", readl (&dev->regs->cap));
	DBG( "otg mux   = %08x\n", readl (&dev->regs->mux));
	DBG( "otg sts   = %08x\n", readl (&dev->regs->sts));
	DBG( "otg ctl   = %08x\n", readl (&dev->regs->ctl));
	DBG( "otg tmr   = %08x\n", readl (&dev->regs->tmr));
	DBG( "otg intr  = %08x\n", readl (&dev->regs->intr));
	DBG( "otg inten = %08x\n", readl (&dev->regs->inten));
	DBG( "--------------------\n");
}
#endif /* DEBUG */

#ifdef CONFIG_SOC_AU1200

/**
 * \brief
 * GPIO functions (Au1200 only)
 *
 * \param  data  pin info
 *
 * \return success
 */
static inline int au1000gpio_bit_get (u32 data)
{
	return (au_readl ((SYS_PINSTATERD & (1 << data)) ? 1 : 0));
}

#ifdef CONFIG_AU1X00_GPIO

	/* GPIO functions are defined in drivers/char/au1000_gpio.c          */

#else

static inline int au1000gpio_bit_tristate (u32 data)
{
	au_writel ((1 << data), SYS_TRIOUTCLR);
	return 0;
}

static inline int au1000gpio_bit_set (u32 data)
{
	au_writel ((1 << data), SYS_OUTPUTSET);
	return 0;
}

static inline int au1000gpio_bit_clear (u32 data)
{
	au_writel ((1 << data), SYS_OUTPUTCLR);
	return 0;
}

#if    LINUX_VERSION_CODE < KERNEL_VERSION(2,5,59)
static inline int au1000gpio_bit_init (u32 data, au1x00_gpio_bit_ctl *opt)
{
	if (opt->direction) {

		if (opt->data) {
			return (au1000gpio_bit_set (data));
		}
		else {
			return (au1000gpio_bit_clear (data));
		}
	}
	else {
		au_writel ((~au_readl (SYS_TRIOUTRD) | (1 << data)),
		           SYS_TRIOUTCLR);
	}
	return 0;
}

static inline int au1000gpio_bit_term (u32 data, au1x00_gpio_bit_ctl *opt)
{
	if (opt->direction) {

		if (opt->data < 0) {
			return (au1000gpio_bit_tristate (data));
		}
		else {
			if (opt->data) {
				return (au1000gpio_bit_set (data));
			}
			else {
				return (au1000gpio_bit_clear (data));
			}
		}
	}
	return 0;
}
#else

static inline int au1000gpio_bit_init (u32 data)
{
	return au1000gpio_bit_set (data);
}

static inline int au1000gpio_bit_term (u32 data)
{
	return au1000gpio_bit_set (data);
}
#endif

static inline int au1000gpio2_bit_set (u32 data)
{
	au_writel (((1 << (data - GPIO_2_BASE)) |
	            (1 << (data - GPIO_2_BASE + 0x10))), GPIO2_OUTPUT);
	au_sync();
	return 0;
}

static inline int au1000gpio2_bit_clear (u32 data)
{
	au_writel ((1 << (data - GPIO_2_BASE + 0x10)), GPIO2_OUTPUT);
	au_sync();
	return 0;
}

static inline int au1000gpio2_bit_init (u32 data)
{
	u32 temp = au_readl (GPIO2_ENABLE);

	if (!(temp | 1)) {
		au_writel (temp | 1, GPIO2_ENABLE);
		au_readl (GPIO2_ENABLE);
		udelay (20);
	}
	if (temp | 2) {
		au_writel (((temp & 0xfffffffd) | 1), GPIO2_ENABLE);
		au_readl (GPIO2_ENABLE);
	}

	/* FIXME: check the system impact of the P0A setting                 */
	/* DbAu1200 pin (GPIO_215) pin is shared w/ PSC0 functions           */
	/* setting this field to GPIO affects also GPIO_16, GPIO_31          */

	if (data == GPIO_215) {
		au_writel (((au_readl (SYS_PINFUNC) &
		             ~((u32) SYS_PINFUNC_P0A)) | SYS_PINFUNC_P0A_GPIO),
			   SYS_PINFUNC);
	}
	au_writel (au_readl (GPIO2_INTENABLE) & ~(1 << (data - GPIO_2_BASE)),
	           GPIO2_INTENABLE);
	au_writel (au_readl (GPIO2_DIR) | (1 << (data - GPIO_2_BASE)),
	           GPIO2_DIR);
	au_sync();
	return 0;
}

static inline int au1000gpio2_bit_term (u32 data)
{
	return au1000gpio2_bit_set (data);
}
#endif
#endif /* CONFIG_SOC_AU1200 */

#endif /* AMD5536UOC_H */
