#include "os.h"
#include "sioll.h"

#if defined(AU1200)
	#define GPIO_DATA	18
	#define GPIO_CLOCK	25
#elif defined(AU1100)
	//#define GPIO_DATA	22
	//#define GPIO_CLOCK	4
	#define GPIO_DATA	4
	#define GPIO_CLOCK	1
#endif

int SIOLL_init_focus(void)
{	
	#if defined(PB1100) | defined(DB1200)
		bcsr->resets |= BCSR_RESETS_TV;
		au_sync();
		bcsr->resets &= ~BCSR_RESETS_PCS0MUX;
		au_sync();

	#elif defined(FICMMP)
		OS_set_config_bits(FICMMP_CONFIG_TVOUTPWREN);	//Enable Focus power
		OS_clear_config_bits(FICMMP_CONFIG_TVODATAOUT);	//Enable data buffers
		//Reset Focus part
		OS_gpio_write(FICMMP_FOCUS_RST, 0);
		OS_mdelay(100);
		OS_gpio_write(FICMMP_FOCUS_RST, 1);
		OS_mdelay(100);
	#endif
			
	#if defined(AU1200)
		OS_gpio_tristate(3);	//Set GP3 to input for external lcd clock
	#endif
	
	return 0;
}

void SIOLL_shutdown_focus(void)
{
	#if defined(FICMMP)
		OS_gpio_tristate(FICMMP_FOCUS_RST);						//Put Focus part in to reset (pull down)
		OS_set_config_bits(FICMMP_CONFIG_TVODATAOUT);	//Disable data buffers
		OS_clear_config_bits(FICMMP_CONFIG_TVOUTPWREN);	//Disable Focus power
	#endif	
}

int SIOLL_init_smbus(void)
{	
	#if defined(AU1200)
		sys->pinfunc |= SYS_PINFUNC_P0B;	//Enable GPIO25 & GPIO18 instead of PSC0
	#endif
	
	OS_gpio_set_inputs();

	return 0;
}

void SIOLL_shutdown_smbus(void)
{
	#if defined(AU1200)
		sys->pinfunc &= ~SYS_PINFUNC_P0B;	//Enable PSC0
	#endif
}

void SIOLL_output_clock(int value)
{
	OS_gpio_write(GPIO_CLOCK, value);
	OS_udelay(5);		//These pullups are pretty weak, allow time to reach 3.3V
	//msdelay(1);
}

void SIOLL_output_data(int value)
{
	if(value)
		OS_gpio_tristate(GPIO_DATA);		//Let the pullup drive the logic '1'
	else
		OS_gpio_write(GPIO_DATA, value);
	
	OS_udelay(5);		//These pullups are pretty weak, allow time to reach 3.3V
}

int SIOLL_input_data(void)
{
	return OS_gpio_read(GPIO_DATA);
}

void SIOLL_set_data_for_output(void)
{
	//This functionality is accomplished when a write occurs
}

void SIOLL_set_data_for_input(void)
{
	OS_gpio_tristate(GPIO_DATA);
	OS_udelay(5);
	//gpio_input_enable(GPIO_DATA);
}


