/*
 *      au1550_i2s_hw.c  --  Sound driver support for AMD Alchemy 
 *   			Au1550/Au1200 MIPS Processor Cores
 *
 */

#ifndef _AU1550_I2S_H
#define _AU1550_I2S_H

#ifdef CONFIG_MIPS_PB1550
#include <asm/mach-pb1x00/pb1550.h>
	#define WM8731
#endif

#ifdef CONFIG_MIPS_DB1550
#include <asm/mach-db1x00/db1x00.h>
	#define WM8731
#endif

#ifdef CONFIG_MIPS_PB1200
#include <asm/mach-pb1x00/pb1200.h>
	#define WM8731
#endif

#ifdef CONFIG_MIPS_DB1200
#include <asm/mach-db1x00/db1200.h>
	#define WM8731
#endif


#define AU1XXX_MODULE_NAME "Au1xxx i2s audio"
#define PFX AU1XXX_MODULE_NAME

#define err(format, arg...) printk(KERN_ERR PFX ": " format "\n" , ## arg)
#define info(format, arg...) printk(KERN_INFO PFX ": " format "\n" , ## arg)

#define supported_mixer(FOO) ((FOO >= 0) && \
                                    (FOO < SOUND_MIXER_NRDEVICES) && \
                                    (s->i2s->input_mask | s->i2s->output_mask) & (1<<FOO) )

enum i2s_power_mode {DAC_OFF=0, ADC_OFF, ALL_OFF, DAC_ON, ADC_ON};

 struct i2s_hw {
	char *name;
	int	  codec_is_master;
	int   input_mask;
	int   output_mask;
	volatile psc_i2s_t *psc;
	unsigned int sr;
	void  (*codec_init)(struct i2s_hw *hw);
	void  (*wrcodec)(u8 ctlreg, u16 val);
	int   (*set_rates)(struct i2s_hw *hw, u32 *in, u32 *out);
	int   (*get_mixer)(struct i2s_hw *hw, u16 oss_channel);
	void  (*set_mixer)(struct i2s_hw *hw, u16 oss_channel, u16 val);
	void (*set_recsrc)(struct i2s_hw *hw, int src);
	int  (*get_recsrc)(void);
#ifdef CONFIG_PM
	  void  (*power)(struct i2s_hw *hw, int type);
#endif

};

	    	  	  

/* Prototypes */
struct i2s_hw * i2s_hw_init(void);
void i2s_hw_cleanup(struct i2s_hw *hw);
extern struct i2s_hw * AMD_PbDb_setup(u32 psc, u32 clock);

/* Helper functions */
static inline void i2s_stop_xmit(struct i2s_hw *hw)
{
	volatile psc_i2s_t *ip = hw->psc;
	u32 stat;

	ip->psc_i2spcr = PSC_I2SPCR_TP;
	au_sync();

	/* Wait for Transmit Busy to show disabled.
	*/
	do {
		stat = ip->psc_i2sstat;
		au_sync();
	} while ((stat & PSC_I2SSTAT_TB) != 0);
}	
	
static inline void i2s_stop_recv(struct i2s_hw *hw)
{
	volatile psc_i2s_t *ip = hw->psc;
	u32 stat;

	ip->psc_i2spcr = PSC_I2SPCR_RP;
	au_sync();

	/* Wait for Transmit Busy to show disabled.
	*/
	do {
		stat = ip->psc_i2sstat;
		au_sync();
	} while ((stat & PSC_I2SSTAT_RB) != 0);
}	

static inline void i2s_start_xmit(struct i2s_hw *hw)
{
	volatile psc_i2s_t *ip = hw->psc;
	ip->psc_i2spcr = PSC_I2SPCR_TC;
	au_sync();
	ip->psc_i2spcr = PSC_I2SPCR_TS;
	au_sync();
}

static inline void i2s_start_recv(struct i2s_hw *hw)
{
	volatile psc_i2s_t *ip = hw->psc;
	ip->psc_i2spcr = PSC_I2SPCR_RC;
	au_sync();
	ip->psc_i2spcr = PSC_I2SPCR_RS;
	au_sync();
}

static inline unsigned i2s_status(struct i2s_hw *hw)
{
	volatile psc_i2s_t *ip = hw->psc;
	u32	i2s_stat;

	i2s_stat = ip->psc_i2sstat;
#ifdef AU1000_VERBOSE_DEBUG
	if (i2s_stat & (PSC_I2SSTAT_TF | PSC_I2SSTAT_TR | PSC_I2SSTAT_TF))
		dbg("I2S status = 0x%08x", i2s_stat);
#endif

	return(i2s_stat);
}

#endif
